** FILE:
**    SWALIB.FOR - part of program SWAP
** FILE IDENTIFICATION:
**    $Id: SWALIB.FOR 1.2.1.17 2002/09/02 09:39:53Z Kroes Exp $
** DESCRIPTION:
**    This file contains program units of SWAP. The program units 
**    in this file are of the 3rd level and are sorted alphabetically
**    with the first letter of the program unit ranging from A - O.
**    The following program units are in this file:
**        ASSIM, ASTRO, DEVRIES, DMCNODE, FLUXES, INITSOL, INTEGRAL, 
**        NOCROP, OUTAFO, OUTAIR, OUTATE, OUTAUN, OUTBAL, 
**        OUTSBA, OUTTEP, OUTVAP, OUTWBA, WATCON
************************************************************************
C ----------------------------------------------------------------------
      SUBROUTINE ASSIM (AMAX,EFF,LAI,KDIF,SINB,PARDIR,PARDIF,FGROS)
C ----------------------------------------------------------------------
C     Author: Daniel van Kraalingen, 1986
C     Calculates the gross CO2 assimilation rate of the whole crop, 
C     FGROS, by performing a Gaussian integration over depth in the 
C     crop canopy. At three different depths in the canopy, i.e. for
C     different values of LAI, the assimilation rate is computed for
C     given fluxes of photosynthetically active radiation, whereafter
C     integration over depth takes place. For more information: see 
C     Spitters et al. (1988). The input variables SINB, PARDIR and 
C     PARDIF are calculated in RADIAT.
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I

      REAL*8  LAI,LAIC,KDIF,KDIRBL,KDIRT
      REAL*8  AMAX,EFF,FGL,FGROS,FGRSH,FGRSUN,FSLLA,GAUSR,PARDIF,PARDIR
      REAL*8  REFH,REFS,SCV,SINB,VISD,VISDF,VISPP,VISSHD,VIST

      DATA    GAUSR /0.3872983d0/
C ----------------------------------------------------------------------
C --- extinction coefficients KDIF,KDIRBL,KDIRT
      SCV    = 0.2d0
      REFH   = (1.0d0-SQRT(1.0d0-SCV))/(1.0d0+SQRT(1.0d0-SCV))
      REFS   = REFH*2.0/(1.0d0+1.6*SINB)
      KDIRBL = (0.5/SINB)*KDIF/(0.8*SQRT(1.0d0-SCV))
      KDIRT  = KDIRBL*SQRT(1.0d0-SCV)

C --- three point gaussian integration over LAI
      FGROS  = 0.
      DO 10 I = 1,3
        LAIC   = 0.5*LAI+GAUSR*(I-2)*LAI
C --- absorbed diffuse radiation (VIDF),light from direct
C --- origine (VIST) and direct light(VISD)
        VISDF  = (1.0d0-REFS)*PARDIF*KDIF  *EXP(-KDIF  *LAIC)
        VIST   = (1.0d0-REFS)*PARDIR*KDIRT *EXP(-KDIRT *LAIC)
        VISD   = (1.0d0-SCV) *PARDIR*KDIRBL*EXP(-KDIRBL*LAIC)
C --- absorbed flux in W/m2 for shaded leaves and assimilation
        VISSHD = VISDF+VIST-VISD
        FGRSH  = AMAX*(1.0d0-EXP(-VISSHD*EFF/AMAX))
C --- direct light absorbed by leaves perpendicular on direct
C --- beam and assimilation of sunlit leaf area
        VISPP  = (1.0d0-SCV)*PARDIR/SINB
        IF (VISPP.LE.0.0d0) FGRSUN = FGRSH
        IF (VISPP.GT.0.0d0) FGRSUN = AMAX*(1.0d0-
     $    (AMAX-FGRSH)*(1.0d0-EXP(-VISPP*EFF/AMAX))/ (EFF*VISPP))
C --- fraction of sunlit leaf area (FSLLA) and local
C --- assimilation rate (FGL)
        FSLLA  = EXP(-KDIRBL*LAIC)
        FGL    = FSLLA*FGRSUN+(1.0d0-FSLLA)*FGRSH
C --- integration
        IF (I.EQ.2) FGL = FGL*1.6
        FGROS  = FGROS+FGL
10    CONTINUE
      FGROS  = FGROS*LAI/3.6

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ASTRO (IDAY,LAT,DAYL,DAYLP,SINLD,COSLD)
C ----------------------------------------------------------------------
C     Author: Daniel van Kraalingen, 1986
C     Computation of astronomical daylength (DAYL), photoperiodic
C     daylength (DAYLP) and two intermediate variables, SINLD and COSLD, 
C     from day number and geographic latitude. Photoperiodic daylength
C     for the effects on development rate is defined as the period that
C     the angle of the sun above the horizon exceeds -4 degrees.
C ----------------------------------------------------------------------
      IMPLICIT NONE
 
      INTEGER IDAY

      REAL*8  LAT
      REAL*8  ANGLE,AOB,COSLD,DAYL,DAYLP,DEC,PI,RAD,SINLD

      DATA    PI /3.1415926d0/,ANGLE /-4.0d0/
C ----------------------------------------------------------------------
C --- declination of the sun as a function of IDAY
      RAD   = PI/180.
      DEC   = -ASIN(SIN(23.45*RAD)*COS(2.*PI*(IDAY+10.0)/365.))
C --- some intermediate variables
      SINLD = SIN(RAD*LAT)*SIN(DEC)
      COSLD = COS(RAD*LAT)*COS(DEC)
      AOB   = SINLD/COSLD      
C --- calculation of daylenght and photoperiodic daylength
      DAYL  = 12.0*(1.0d0+2.*ASIN(AOB)/PI)
      DAYLP = 12.0*(1.0d0+2.*ASIN((-SIN(ANGLE*RAD)+SINLD)/COSLD)/PI)

      RETURN
      END

************************************************************************
      Subroutine Devries (NumNod,theta,THETAS,HeaCap,HeaCon,FQUARTZ,
     &           FCLAY,FORG)
************************************************************************
** Purpose:    Calculate soil heat capacity and conductivity for each  *
**             compartment by full de Vries model                      *     
** References:                                                         *
** Description:                                                        *
** de Vries model for soil heat capcity and thermal conductivity.      * 
** Heat capacity is calculated as average of heat capacities for each  *
** soil component. Thermal conductivity is calculated as weighted      *
** average of conductivities for each component. If theta > 0.05 liquid*
** water is assumed to be the main transport medium in calculating the *
** weights. If theta < 0.02 air is assumed to be the main transport    *
** medium (there is also an empirical adjustment to the conductivity). *
** For 0.02 < theta < 0.05 conductivity is interpolated.               *
** See: Heat and water transfer at the bare soil surface, H.F.M Ten    *
** Berge (pp 48-54 and Appendix 2)                                     *
************************************************************************
** Input:                                                              *
** NumNod - number of compartments (-)                                 *
** theta/THETAS - volumetric soil moisture/ saturated vol. s. moist (-)*
** Fquartz, Fclay and Forg - volume fractions of sand, clay and org.ma.*
** Output:                                                             *
** HeaCap - heat capacity (J/m3/K)                                     *
** HeaCon - thermal conductivity (W/m/K)                               *
************************************************************************
      Implicit None
      Include 'Param.fi'
  
*     (i) Global declarations                                          
*     (i.i) Input                                                      
      Integer NumNod
      Real*8 theta(1:MACP),THETAS(1:MACP)
*     (i.ii)
      Real*8 HeaCap(MACP), HeaCon(MACP)
*     (ii) Local declarations                                          
      Integer Node
      Real*8 kqa,kca,kwa,koa,kaa,kqw,kcw,kww,kow,kaw
      Parameter (kaa = 1.0d0, kww = 1.0d0)
      Real*8 fAir(MACP),fClay(MACP),fQuartz(MACP),fOrg(MACP)
      Real*8 HeaConDry,HeaConWet
*     (iii) Parameter declarations                                     
*     (iii.i) Physical constants                                       
*     Specific heats (J/kg/K)                                          
      REAL*8  cQuartz,cClay,cWat,cAir,cOrg
      Parameter (cQuartz = 800.0d0, cClay = 900.0d0, cWat = 4180.0d0,
     &           cAir = 1010.0d0, cOrg = 1920.0d0)
*     Density (kg/m)
      Real*8 dQuartz, dClay, dWat, dAir, dOrg
      Parameter (dQuartz = 2660.0d0, dClay = 2650.0d0, dWat = 1000.0d0,
     &           dAir = 1.2d0, dOrg = 1300.0d0)
*     Thermal conductivities (W/m/K)                                   
      REAL*8  kQuartz,kClay,kWat,kAir,kOrg
      Parameter (kQuartz = 8.8d0, kClay = 2.92d0, kWat = 0.57d0,
     &           kAir = 0.025d0, kOrg = 0.25d0)
*     
      REAL*8  GQuartz,GClay,GWat,GAir,GOrg
      Parameter (GQuartz = 0.14d0, GClay = 0.0d0 , GWat = 0.14d0, 
     &           GAir = 0.2d0, GOrg = 0.5d0)
*     (iii.ii) theta 0.02, 0.05 and 1.00                               
      REAL*8thetaDry,thetaWet
      Parameter (thetaDry = 0.02d0, thetaWet = 0.05d0)    
C ----------------------------------------------------------------------
*     (0) Weights for each component in conductivity calculations      
*     (Calculate these but define as parameters in later version)     
      kaw = 0.66 / (1.0d0 + ((kAir/kWat) - 1.0d0) * GAir) + 0.33d0 /
     &      (1.0d0 + ((kAir/kWat) - 1.0d0) * (1.0d0 - 2.0 * GAir)) 
      kqw = 0.66 / (1.0d0 + ((kQuartz/kWat) - 1.0d0) * GQuartz)+0.33d0 /
     &      (1.0d0 + ((kQuartz/kWat) - 1.0d0) * (1.0d0 - 2.0 * GQuartz)) 
      kcw = 0.66 / (1.0d0 + ((kClay/kWat) - 1.0d0) * GClay) + 0.33d0 /
     &      (1.0d0 + ((kClay/kWat) - 1.0d0) * (1.0d0 - 2.0 * GClay)) 
      kow = 0.66 / (1.0d0 + ((kOrg/kWat) - 1.0d0) * GOrg) + 0.33d0 /
     &      (1.0d0 + ((kOrg/kWat) - 1.0d0) * (1.0d0 - 2.0 * GOrg)) 
      kwa = 0.66 / (1.0d0 + ((kWat/kAir) - 1.0d0) * GWat) + 0.33d0 /
     &      (1.0d0 + ((kWat/kAir) - 1.0d0) * (1.0d0 - 2.0 * GWat)) 
      kqa = 0.66 / (1.0d0 + ((kQuartz/kAir) - 1.0d0) * GQuartz)+0.33d0 /
     &      (1.0d0 + ((kQuartz/kAir) - 1.0d0) * (1.0d0 - 2.0 * GQuartz)) 
      kca = 0.66 / (1.0d0 + ((kClay/kAir) - 1.0d0) * GClay) + 0.33d0 /
     &      (1.0d0 + ((kClay/kAir) - 1.0d0) * (1.0d0 - 2.0 * GClay)) 
      koa = 0.66 / (1.0d0 + ((kOrg/kAir) - 1.0d0) * GOrg) + 0.33d0 /
     &      (1.0d0 + ((kOrg/kAir) - 1.0d0) * (1.0d0 - 2.0 * GOrg)) 

      Do Node = 1,NumNod

*        (1) Air fraction
         fAir(Node) = THETAS(Node) - theta(Node)

*        (2) Heat capacity (W/m3/K) is average of heat capacities for  
*        all components (multiplied by density for correct units)      
         HeaCap(Node) = fQuartz(Node)*dQuartz*cQuartz + fClay(Node)*
     &                  dClay*cClay + theta(Node)*dWat*cWat + 
     &                  fAir(Node)*dAir*cAir +
     &                  fOrg(Node)*dOrg*cOrg

*        (3) Thermal conductivity (W/m/K) is weighted average of       
*        conductivities of all components                              
*        (3.1) Dry conditions (include empirical correction) (eq. 3.44)
         If (theta(Node).LE.thetaDry) Then
            HeaCon(Node) = 1.25 * (kqa*fQuartz(Node)*kQuartz + 
     &                      kca*fClay(Node)*kClay +
     &                      kaa*fAir(Node)*kAir +
     &                      koa*fOrg(Node)*kOrg +
     &                      kwa*theta(Node)*kWat) /
     &   (kqa * fQuartz(Node) + kca * fClay(Node) + kaa * fAir(Node) +
     &    koa * fOrg(Node) + kwa * theta(Node))

*        (3.2) Wet conditions  (eq. 3.43)                              
         Else If (theta(Node).GE.thetaWet) Then
            HeaCon(Node) = (kqw*fQuartz(Node)*kQuartz + 
     &                      kcw*fClay(Node)*kClay +
     &                      kaw*fAir(Node)*kAir +
     &                      kow*fOrg(Node)*kOrg +
     &                      kww*theta(Node)*kWat) /
     &   (kqw * fQuartz(Node) + kcw * fClay(Node) + kaw * fAir(Node) +
     &    kow * fOrg(Node) + kww * theta(Node))

*        (3.3) dry < theta < wet (interpolate)
         Else
*           (3.3.1) Conductivity for theta = 0.02                      
            HeaConDry = 1.25 * (kqa*fQuartz(Node)*kQuartz + 
     &                      kca*fClay(Node)*kClay +
     &                      kaa*fAir(Node)*kAir +
     &                      koa*fOrg(Node)*kOrg +
     &                      kwa*thetaDry*kWat) /
     &   (kqa * fQuartz(Node) + kca * fClay(Node) + kaa * fAir(Node) +
     &    koa * fOrg(Node) + kwa * thetaDry)
*           (3.3.1) Conductivity for theta = 0.05                      
            HeaConWet = (kqw*fQuartz(Node)*kQuartz + 
     &                      kcw*fClay(Node)*kClay +
     &                      kaw*fAir(Node)*kAir +
     &                      kow*fOrg(Node)*kOrg +
     &                      kww*thetaWet*kWat) /
     &   (kqw * fQuartz(Node) + kcw * fClay(Node) + kaw * fAir(Node) +
     &    kow * fOrg(Node) + kww * thetaWet)
*         (3.3.3) Interpolate                                          
          HeaCon(Node) = HeaConDry + (theta(Node)-thetaDry) * 
     &                   (HeaConWet - HeaConDry) 
     &                 / (thetaWet - thetaDry)      
         End If

C ---    conversion of capacity from J/m3/K to J/cm3/K
         HEACAP(NODE) = HEACAP(NODE)*1.0d-6

C ---    conversion of conductivity from W/m/K to J/cm/K/d
         HEACON(NODE) = HEACON(NODE)*864.0

      End Do

      Return
      End 

C ----------------------------------------------------------------------
      SUBROUTINE FLUXES (Q,QBOT,DT,INQ,NUMNOD,THETM1,THETA,DZ,QROT,
     & QDRA,QIMMOB,ADSFLU,QTOP,QROSUM,QDRTOT,VOLACT,VOLM1,SWBOTB,
     & QCRACK,FLREVA,POND,WTOPLAT,ARFLUX,NIRD,ACRACK,REVA,QCOMP,EVP)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculates the fluxes between compartments
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,SWBOTB

      REAL*8  QBOT,DT,INQ(MACP+1),DZ(MACP),QROT(MACP),QDRA(5,MACP)
      REAL*8  QIMMOB(MACP),ADSFLU(MACP),QTOP,QROSUM,QDRTOT,QCRACK
      REAL*8  Q(MACP+1),THETM1(MACP),THETA(MACP),VOLACT,VOLM1
      REAL*8  POND,WTOPLAT,ARFLUX,NIRD,ACRACK,REVA,QCOMP
      LOGICAL FLREVA
      REAL*8  EVP(MACP)


C ----------------------------------------------------------------------
C --- local
      INTEGER i

C ----------------------------------------------------------------------

C --- determine qbot if not specified
      if (swbotb .eq. 1 .or. swbotb .eq. 5 .or. swbotb .eq. 8) then
        qbot = qtop + qrosum + qdrtot - qcrack + (volact-volm1)/dt
      endif

C --- calculate fluxes (cm/d) from changes in volume per compartment
      i      = numnod+1
      q(i)   = qbot
      inq(i) = inq(i) + q(i)*dt
      do 100 i = numnod,1,-1
        q(i) = - (theta(i)-thetm1(i)+qimmob(i))*dz(i)/dt + q(i+1)-
     &                qrot(i)-qdra(1,i)-qdra(2,i)-qdra(3,i)-qdra(4,i)-
     &                qdra(5,i)+adsflu(i)
        inq(i) = inq(i) + q(i)*dt
100   continue

c --- determine reduced evaporation rate in case of FLREVA
      if (flreva) reva = max(0.0d0,qtop + (pond - wtoplat)/dt - qcomp
     &                   + (arflux+nird)*(1.0d0 - acrack))

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE INITSOL(
     & NUMLAY,FLGENU,THETHI,THETLO,HTABLE,KTABLE,THETSL,
     & KSAT,DMCH,DMCC,COFGEN,KSE99,NUMNOD,DZ,Z,
     & DISNOD,INPOLB,INPOLA,BOTCOM,LAYER,THETAS,THETAR,SWHYST,INDEKS,
     & ALFAMG,SWINCO,THETA,THETAI,H,SWBOTB,HGITAB,T,HI,GWLTAB,GWLI,GWL,
     & GWLBAK,SWSOLU,CMLI,CML,CIL,SAMINI,PSAND,PSILT,PCLAY,ORGMAT,CL,
     & CMSY,KF,FREXP,CISY,VOLM1,VOLACT,VOLINI,VOLSAT,CREF,THETM1,HM1,
     & DIMOCA,K,KMEAN,THETIM,SWMOBI,FMOBIL,KMOBIL,PF1,FM1,PF2,FM2,
     & SLOPFM,QIMMOB,SWCRACK,SHRINA,MOISR1,MOISRD,ZNCRACK,SHRINB,SHRINC,
     & NNCRACK,ACRACK,QCRACK,RAPDRA,WTOPLAT,ADSFLU,CKWM1,DT,
     & POND,BDENS,FQUARTZ,FCLAY,FORG)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : initialize soil profile data
C     Subroutines called : RDSOL,WATCON
C     Functions called   : PRHNODE,THENODE,DMCNODE,HCONODE,AFGEN
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER THETHI(MAHO),THETLO(MAHO),BOTCOM(MAHO),LAYER(MACP)
      INTEGER NUMLAY,NUMNOD,SWHYST,SWINCO,SWBOTB,SWSOLU,NNCRACK
      INTEGER SWMOBI,SWCRACK, INDEKS(MACP)

      REAL*8 HGITAB(732),T,GWLTAB(732),GWL,KF,SHRINA,MOISR1,MOISRD
      REAL*8 SHRINB,SAMINI,SHRINC,ACRACK,QCRACK,RAPDRA,WTOPLAT,KMOBIL
      REAL*8 AFGEN,FREXP,CKWM1,GWLI,CREF,GWLBAK(4),POND,DT,ARFLUX,NIRD
      REAL*8 CRACKW,ZNCRACK
      REAL*8 FMOBIL(MACP),DZ(MACP),Z(MACP),DISNOD(MACP+1),INPOLA(MACP)
      REAL*8 INPOLB(MACP),CMLI(MACP),CML(MACP),CIL(MACP),CL(MACP)
      REAL*8 CMSY(MACP),CISY(MACP),PF1(MAHO),FM1(MAHO),SLOPFM(MAHO)
      REAL*8 ADSFLU(MACP),PF2(MAHO),FM2(MAHO),QIMMOB(MACP),PCLAY(MAHO)
      REAL*8 PSAND(MAHO),BDENS(MAHO)
      REAL*8 PRHNODE,THENODE,DMCNODE,HCONODE,VOLM1,VOLACT,VOLINI,VOLSAT
      REAL*8 HTABLE(MAHO,99),KTABLE(MAHO,99),THETSL(MAHO),KSAT(MAHO)
      REAL*8 DMCC(MAHO,99),DMCH(MAHO,99),COFGEN(8,MAHO),KSE99(MAHO)
      REAL*8 THETAS(MACP),THETAR(MACP),ALFAMG(MACP),THETA(MACP)
      REAL*8 THETAI(MACP),H(MACP),HI(MACP),THETM1(MACP),HM1(MACP)
      REAL*8 DIMOCA(MACP),K(MACP+1),KMEAN(MACP+1),THETIM(MAHO)

      LOGICAL FLGENU(MAHO)

      REAL*8 PSILT(MAHO),FQUARTZ(MACP),FCLAY(MACP),FORG(MACP)
      REAL*8 ORGMAT(MAHO)

C ----------------------------------------------------------------------
C --- local
      INTEGER LAY,NODE,I,I2
      REAL*8  beta1,beta2,func,deri
      REAL*8 DUMMY,GMINERAL

C --- Variables for error handling
      Character*80 ErrTxt,ErrSrc
	Integer GetErrorStatus
	External GetErrorStatus

      save
C ----------------------------------------------------------------------

C --- Define source (routine name) of potential error
      ErrSrc = 'InitSol'

C --- position of nodal points and distances between them
      Z(1)      = - 0.5*DZ(1)
      DISNOD(1) = - Z(1)
      do 20 NODE=2,NUMNOD
        Z(NODE)      = Z(NODE-1) - 0.5*(DZ(NODE-1)+DZ(NODE))
        DISNOD(NODE) = Z(NODE-1)-Z(NODE)
 20   continue
      DISNOD(NUMNOD+1) = 0.5*DZ(NUMNOD)

C --- interpolation values for calculation solute transport
      INPOLB(1) = 0.5*DZ(1)/DISNOD(2)
      do 22 NODE = 2,NUMNOD-1
        INPOLA(NODE) = 0.5*DZ(NODE)/DISNOD(NODE)
        INPOLB(NODE) = 0.5*DZ(NODE)/DISNOD(NODE+1)
22    continue
      INPOLA(NUMNOD) = 0.5*DZ(NUMNOD)/DISNOD(NUMNOD)

c --- immobile soil fractions
      if (SWMOBI.eq.1) then
        do 24 I = 1,NUMLAY
          SLOPFM(I) = (FM2(I)-FM1(I))/(PF2(I)-PF1(I))
   24   continue
      else
        do 26 I = 1,NUMNOD
          FMOBIL(I) = 1.0d0
   26   continue
        KMOBIL    = 0.0
      endif
      do 28 I = 1,NUMNOD
        QIMMOB(I) = 0.0
   28 continue

C --- per node: layer, saturated watercontent, dry bulk density
C --- volume fractions sand, clay and organic matter, hysteresis data
      LAY = 1
      do 40 NODE = 1,NUMNOD
        if (NODE.gt.BOTCOM(LAY)) LAY = LAY+1
        LAYER(NODE) = LAY
        if (FLGENU(LAY)) then
          THETAS(NODE) = COFGEN(2,LAY)
          THETAR(NODE) = COFGEN(1,LAY)
          if (SWHYST.eq.1) then

C --- wetting curve  
            INDEKS(NODE)  = 1
            ALFAMG(NODE)  = COFGEN(8,LAY)
          elseif (SWHYST.eq.0.OR.SWHYST.eq.2) then

C --- drying branch or simulation without hysteresis
            INDEKS(NODE)   = -1
            ALFAMG(NODE)  = COFGEN(4,LAY)
          endif
        else
          THETAS(NODE) = THETSL(LAY)
        endif

        DUMMY = ORGMAT(LAY)/(1.0d0 - ORGMAT(LAY))
        GMINERAL = (1.0d0 - THETAS(NODE)) / (0.370d0 + 0.714*DUMMY)
        BDENS(LAY) = (1.0d0 + DUMMY) * GMINERAL
        FQUARTZ(NODE) = (PSAND(LAY) + PSILT(LAY))*GMINERAL/2.7
        FCLAY(NODE) = PCLAY(LAY)*GMINERAL/2.7
        FORG(NODE) = DUMMY*GMINERAL/1.4
 40   continue

C --- Set initial conditions
      POND = 0.0

      if (SWINCO.eq.0) then

C --- compute initial pressure heads from moisture content
        do 60 NODE = 1,NUMNOD
          THETA(NODE)  = THETAI(NODE)
          H(NODE) = PRHNODE (NODE,THETA(NODE),LAYER,DISNOD,H,
     &    FLGENU,THETAR,THETAS,COFGEN,ALFAMG,HTABLE)
 60     continue

C --- change h for last node
        if (SWBOTB.eq.5) then
          H(NUMNOD)     = AFGEN (HGITAB,732,T+1.0d0)
          THETA(NUMNOD) = THENODE (NUMNOD,H(NUMNOD),LAYER,
     &    FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
        endif
      elseif (SWINCO.eq.1) then

C --- pressure head profile is given 
        do 70 NODE = 1,NUMNOD
          H(NODE) = HI(NODE)
   70   continue

C --- in case of preferential flow, adjust MVG parameters
        if (SWMOBI.eq.1) then
          do 170 i = 1,numnod
            lay = layer(i)
            if (h(i).gt.-1.0d0) then
              fmobil(i) = fm1(lay)+(0.0d0-pf1(lay))*slopfm(lay)
            else
              fmobil(i) = fm1(lay)+(log10(abs(h(i)))-pf1(lay))*
     &                    slopfm(lay)
            endif
            fmobil(i) = min (1.0d0,fmobil(i))
            fmobil(i) = max (0.3d0,fmobil(i))
            thetar(i) = cofgen(1,lay)*fmobil(i)
            thetas(i) = cofgen(2,lay)*fmobil(i)
 170      continue
        endif
        do 180 node = 1,numnod
          THETA(NODE) = THENODE(NODE,H(NODE),LAYER,FLGENU,
     &                  COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
 180    continue
      elseif (SWINCO.eq.2) then

C --- pressure head profile is calculated 
        if (SWBOTB.eq.1) then  
          GWL = AFGEN (GWLTAB,732,T+1.0d0)
        else
          GWL = GWLI
        endif
        if (GWL.gt.0.0) POND = GWL
        do 80 I = 1,NUMNOD
          H(I) = GWL - Z(I)

C --- in case of preferential flow, adjust MVG parameters
          if (SWMOBI.eq.1) then
            lay = layer(i)
            if (h(i).gt.-1.0d0) then
              fmobil(i) = fm1(lay)+(0.0d0-pf1(lay))*slopfm(lay)
            else
              fmobil(i) = fm1(lay)+(log10(abs(h(i)))-pf1(lay))*
     &                    slopfm(lay)
            endif
            fmobil(i) = min (1.0d0,fmobil(i))
            fmobil(i) = max (0.3d0,fmobil(i))
            thetar(i) = cofgen(1,lay)*fmobil(i)
            thetas(i) = cofgen(2,lay)*fmobil(i)
          endif
          THETA(I) = THENODE (I,H(I),LAYER,FLGENU,
     &               COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
 80     continue 
      endif

C --- water volume in totally saturated profile
      VOLSAT = 0.0
      do 82 NODE = 1,NUMNOD
        VOLSAT = VOLSAT + DZ(NODE)*THETSL(LAYER(NODE))
 82   continue 

C --- save initial moisture contents and pressure heads
      do 90 NODE = 1,NUMNOD
        THETM1(NODE) = THETA(NODE)
        HM1(NODE)    = H(NODE)
 90   continue

C --- hydr. conductivities, differential moisture capacities
C     and geometrical mean hydr. conductivities for each node
      DIMOCA(1) = DMCNODE(1,H(1),LAYER,FLGENU,COFGEN,THETLO,THETHI,
     &            DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,
     &            PF1,SLOPFM,HTABLE,THETA)
      K(1) = HCONODE (1,THETA(1),LAYER,FLGENU,COFGEN,THETAR,THETAS,
     &       THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
      do 50 NODE = 2,NUMNOD
        DIMOCA(NODE) = DMCNODE (NODE,H(NODE),LAYER,FLGENU,COFGEN,
     &                 THETLO,THETHI,DMCH,DMCC,THETAS,THETAR,
     &                 ALFAMG,SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
        K(NODE) = HCONODE (NODE,THETA(NODE),LAYER,FLGENU,COFGEN,
     &            THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,
     &            SWMOBI,FMOBIL)
        KMEAN(NODE) = 0.5*(K(NODE)+K(NODE-1))
	  IF(GetErrorStatus().ne.0) RETURN
 50   continue
      KMEAN(NUMNOD+1) = K(NUMNOD)

C --- initial solute profiles
      if (SWSOLU.eq.1) then
        SAMINI = 0.0
        do 4 I = 1,NUMNOD
          LAY = LAYER(I)
          if (FMOBIL(I).gt.0.995d0) then
            CML(I) = CMLI(I)
            CIL(I) = 0.0   
            CL(I)  = CML(I)
            CMSY(I) = (THETA(I)*CML(I)+BDENS(LAY)*KF*CREF*(CML(I)
     &                /CREF)**FREXP)
            CISY(I) = 0.0
          else
            CML(I) = CMLI(I)
            CIL(I) = CML(I)
            CL(I)  = CML(I)
            CMSY(I) = (THETA(I)*CML(I)+BDENS(LAY)*KF*CREF*((CML(I)
     &                /CREF)**FREXP) * FMOBIL(I))
            CISY(I) = ((THETIM(LAY)*CIL(I)+BDENS(LAY)*KF*CREF
     &                *(CIL(I)/CREF)**FREXP)* (1.0d0-FMOBIL(I)))
          endif
          SAMINI = SAMINI + (CMSY(I)+CISY(I)) * DZ(I)
 4      continue
      endif

C --- flow through soil cracks
      if (SWCRACK.eq.1) then
        shrinc = 0.7d0
        beta2  = -1.0/moisr1*log((moisrd+moisr1-shrinc*moisr1)/shrina)      
        beta1  = beta2+1.0d0

 445    if (abs(beta2-beta1).gt.0.001d0) then
          beta1 = beta2
          func  = (shrina+shrina*moisr1*beta1)*exp(-beta1*moisr1)-moisrd
          deri  = (2.0*shrina*moisr1-shrina*moisr1*moisr1*beta1)*
     &            exp(-beta1*moisr1)
          beta2 = beta1-func/deri
          goto 445
        endif
        shrinb = beta2
        shrinc = 1.0d0+shrina*shrinb*exp(-shrinb*moisr1) 

C --- number node for evaluating shrinkage characteristic
        i = 1
 460    if (z(i).gt.zncrack) then
          i = i+1
          goto 460
        endif
        nncrack = i
      else

C ---   set variables to zero
        acrack  = 0.0
        qcrack  = 0.0
        rapdra  = 0.0
        wtoplat = 0.0
        do 443 i = 1,numnod
          adsflu(i) = 0.0
  443   continue
      endif         

C --- initial total volume
      ARFLUX = 0.0
      NIRD   = 0.0
      CRACKW = 0.0

C --- call WATCON (WATer CONtent)
      call WATCON (VOLM1,VOLACT,NUMNOD,THETA,DZ,FMOBIL,LAYER,
     & THETIM,SWCRACK,CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX,NIRD,ACRACK,
     & RAPDRA,DT)

      VOLINI = VOLACT+POND

C --- initialize registration of last four groundwaterlevels, for noting
C --- oscillation
      do 454 i2=1,4
        gwlbak(i2)= 0.0
 454  continue

      return
      end
C ----------------------------------------------------------------------
      SUBROUTINE INTEGRAL (PTRA,DT,PEVA,REVA,QBOT,POND,QTOP,NRAI,NIRD,
     & PONDMX,NUMNOD,QROT,QDRTOT,QDRAIN,DQROT,DPTRA,IQROT,INQROT,IQDRA,
     & NRLEVS,INQDRA,QDRA,GRAI,GIRD,IINTC,IPTRA,IPEVA,IEVAP,IRUNO,
     & IPREC,IQBOT,CQROT,CQDRA,CQDRAIN,CPTRA,CPEVA,CEVAP,CRUNO,
     & CGRAI,CNRAI,CGIRD,CNIRD,CQBOT,PONDM1,WTOPLAT,RAPDRA,
     & IQDRAR,IQCRACK,QCRACK,CQDRAR,CQCRACK,ACRACK,ARFLUX,T,
     & WLS,RSRO,RUNOTS,QROSUM,SWCRACK,SWDRA,IGRAI,IGIRD,INRAI,INIRD,
     & QCOMP,VOLINI,VOLACT,CRACKW,WBALANCE,SWBALANCE)
C ----------------------------------------------------------------------
C     Date               : 6/6/2001                                        
C     Purpose            : calculation of intermediate values and     
C                          cumulative values          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,NRLEVS,SWCRACK,SWDRA,SWBALANCE

      REAL*8  PTRA,DT,PEVA,REVA,POND,QTOP,NRAI,NIRD,PONDMX,QROT(MACP)
      REAL*8  QDRTOT,QDRAIN(5),DQROT,DPTRA,IQROT,INQROT(MACP),IQDRA
      REAL*8  INQDRA(5,MACP),IGRAI,IGIRD
      REAL*8  QDRA(5,MACP),IINTC,IPTRA,IPEVA,IEVAP,IRUNO,IPREC
      REAL*8  IQBOT,GRAI,GIRD,CQROT,CQDRA,CQDRAIN(5),CPTRA,CPEVA,CEVAP
      REAL*8  CRUNO,QROSUM,QCOMP,T
      REAL*8  CGRAI,CNRAI,CGIRD,CNIRD,CQBOT,QBOT,ACRACK
      REAL*8  PONDM1,WTOPLAT,RAPDRA,IQDRAR,IQCRACK,QCRACK,CQDRAR,CQCRACK
      REAL*8  WLS,RSRO,RUNOTS,ARFLUX ,INRAI,INIRD,Volini,Volact,Crackw        
      
C ----------------------------------------------------------------------
C --- local
      INTEGER node,level

      REAL*8  qrotts,qdrats,ptrats,pevats,revats,masdif
      REAL*8  qbotts,difpond,wbalance
C ----------------------------------------------------------------------
              
C --- potential transpiration of this timestep
      ptrats = ptra * dt

C --- potential soil evaporation of this timestep
      pevats = peva * dt

C --- reduced soil evaporation of this timestep
      revats = reva * dt

C --- flux lower boundary of this timestep
      qbotts = qbot*dt

C --- runoff of this timestep
      pondm1 = pond
      difpond = (qtop + (ARFLUX + NIRD) * (1.0d0 - acrack)) * dt 
     &        - revats - wtoplat
      pond = pond + difpond
      if (pond.lt.1.0d-6) pond=0.0

C --- height ponding layer determines amount runoff or runon
      runots = 0.0
      if (SWDRA.LT.2) then
        if (pond.gt.pondmx .and. .not. (SWCRACK.EQ.1)) then
            runots = pond - pondmx
        endif
      else
         if (pond.gt.pondmx .or. wls.gt.pondmx) then
           if (pond.gt.pondmx) then
             if (wls.gt.pondmx) then
                runots = pond - wls
             else
                runots = pond - pondmx
             endif
           else
             runots = pondmx - wls
           endif
C ---      reduce runots so that the rate corresponds to a drainage flux
C ---      with drainage resistance of RSRO
           runots = runots*dt/rsro
         endif
      endif
      pond = pond - runots


C --- total root extraction of this timestep
      qrotts = QROSUM * dt

C --- total drainage flux of this timestep
      qdrats = qdrtot * dt

C --- add time step fluxes to daily totals
      dqrot = dqrot + qrotts
      dptra = dptra + ptrats

C --- add time step fluxes to intermediate totals
      iqrot  = iqrot + qrotts
      do 300 node = 1,numnod
        inqrot(node) = inqrot(node) + qrot(node) * dt
  300 continue
      iqdra = iqdra + qdrats + rapdra*dt
      do 410 node = 1,numnod
        do 400 level = 1,nrlevs
          inqdra(level,node) = inqdra(level,node)+qdra(level,node)*dt
400     continue
410   continue

      iintc   = iintc + ((GRAI-NRAI)+(GIRD-NIRD))*dt
      iptra   = iptra + ptrats
      ipeva   = ipeva + pevats
      ievap   = ievap + revats
      iruno   = iruno + runots
      iprec   = iprec + (GRAI+GIRD)*dt
      igrai   = igrai + grai*dt
      igird   = igird + gird*dt
      inrai   = inrai + nrai*dt
      inird   = inird + nird*dt
      iqbot   = iqbot + qbotts
      iqdrar  = iqdrar + rapdra*dt 
      iqcrack = iqcrack +qcrack*dt

C --- add time step fluxes to total cumulative values
      cqrot   = cqrot + qrotts
      cqdra   = cqdra + qdrats
      cptra   = cptra + ptrats
      cpeva   = cpeva + pevats
      cevap   = cevap + revats
      cruno   = cruno + runots
      CGRAI   = CGRAI + GRAI*DT
      CNRAI   = CNRAI + ARFLUX*DT
      CGIRD   = CGIRD + GIRD*DT
      CNIRD   = CNIRD + NIRD*DT
      cqbot   = cqbot + qbotts
      cqdrar  = cqdrar + rapdra*dt
      cqcrack = cqcrack + qcrack*dt 
      do 150 level = 1,5
        cqdrain(level) = cqdrain(level) + qdrain(level)*dt
150   continue

c --- compensate water balance error of this time step during remaining day part
c --- cumulative water balance error
      wbalance = cnrai + cnird - cruno - cqrot - cevap - cqdra - cqdrar
     &         + cqbot + volini - volact - pond - crackw

c --- in case of ponding, correct ponding height with water balance error
      if (pond .gt. 1.0d-8 .and. swbalance .eq. 1) then
        if ((wbalance+pond).gt.pondmx .and. swdra.lt.2) then
           wbalance = wbalance - (pondmx - pond)
           pond = pondmx
        else
          if (wbalance + pond .lt. 0.0d0) then
            wbalance = wbalance + pond
            pond = 0.0
          else
            pond = pond + wbalance
            wbalance = 0.0
          endif
        endif
      endif

c --- adjust remaing water balance error in qtop with qcomp(ensation)
c --- during remaining time of this day
      if (swbalance .eq. 1) qcomp =  - wbalance / (aint(t+1.0d0) - t)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE NOCROP (DRZ,LAI,CF,FLCROP)
C ----------------------------------------------------------------------
      REAL*8  DRZ,LAI,CF
      LOGICAL FLCROP
C ----------------------------------------------------------------------
      DRZ    = 0.0
      LAI    = 0.0
      CF     = 0.0
      FLCROP = .FALSE.

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTAFO (AFO,AFONAM,HAFOD,ENVSTR,
     &  NUMNOD,OUTPER,PERIOD,TCUM,POND,GWL,THETA,H,INQ,
     &  INQROT,NRLEVS,INQDRA,IEVAP,IPEVA,IPTRA,IRUNO,
     &  NUMLAY,BOTCOM,DZ,THETAS,BRUNY,ERUNY,BRUND,ERUND,
     &  LAYER,FLGENU,ALFAMG,THETAR,COFGEN,HTABLE,THETLO,
     &  SWAFO,TEMPI,IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV,TSoil,DAYNRC,
     &  DRZ,CF,WBALANCE,prjdes)
C ----------------------------------------------------------------------
C     Date               : 6/6/01           
C     Purpose            : ANIMO output: formatted hydrological data
C     Subroutines called : -              
C     Functions called   : THENODE 
C     File usage         : AFONAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      integer*4 getun,ftype

      INTEGER AFO,OUTPER,PERIOD,LAY,LEVEL,NODE,DayNrC
      INTEGER NUMLAY,NUMNOD,NRLEVS,BOTCOM(MAHO),LAYER(MACP)
      INTEGER BRUNY,ERUNY,BRUND,ERUND,THETLO(MAHO),SWAFO

      REAL*8  GWL,INQDRA(5,MACP),IPREC,IINTC,IEVAP,POND,TCUM,DRZ,CF
      REAL*8  IPEVA,IPTRA,IRUNO,INQ(MACP+1),INQROT(MACP),DZ(MACP)
      REAL*8  TEMPI(MACP),IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV 
      REAL*8  THETA(MACP),H(MACP),THETAS(MACP),THENODE
      REAL*8  ALFAMG(MACP),THETAR(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  DVAL(MAHO),TSoil(0:MACP),WBALANCE

      CHARACTER AFONAM*16,ENVSTR*50,FILNAM*80, AFOEXT*4
      CHARACTER*80 prjdes

      LOGICAL HAFOD,FLGENU(MAHO)
C ----------------------------------------------------------------------
      IF (.NOT.HAFOD) THEN

C ---   open file
        if (swafo.eq.1) then
          afoext = '.afo'
        elseif (swafo.eq.2) then
          afoext = '.pfo'
        endif
        filnam = trim(envstr)//trim(afonam)//afoext
        afo = getun (20,30)
        call fopens(afo,filnam,'new','del')


C ---   write initial part
        if (swafo.eq.2) then
          ftype = 1
          call writehead(afo,ftype,prjdes)
        endif
        if (period.eq.0) period = -10
        write (AFO,4000) BRUNY,ERUNY, 
     &  float(BRUND-1),float(ERUND),float(PERIOD)
        if (period.eq.-10) period = 0

        write (AFO,4010) numnod,numlay,nrlevs
        write (AFO,4010) (botcom(lay),lay=1,numlay)
        write (AFO,4020) (real(thetaS(botcom(lay))), lay=1,numlay)

        DO 4 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-100.d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
4       CONTINUE
        write (AFO,4020) (REAL(DVAL(lay)),lay=1,numlay) 

        DO 6 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-15849.0d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
6       CONTINUE
        write (AFO,4020) (REAL(DVAL(lay)),lay=1,numlay) 

        write (AFO,4020) (0.01*real(dz(node)),node=1,numnod)
        write (AFO,4020) (real(theta(node)),node=1,numnod)
        write (AFO,4020) -0.01*real(gwl),0.01*real(pond)
        If (SWAFO.EQ.2) then
          write (AFO,4020) (real(tempi(node)),node = 1,numnod) 
        endif
4000    format (2(1X,I8),3(1X,F8.0),1X,i8) 
4010    format (5(1X,i8))
4020    format (8(1X,F8.4))

        HAFOD = .TRUE.
        RETURN
      ENDIF

C --- formatted output of data - dynamic part
      if (HAFOD) then
        if (SWAFO.eq.1) then
          write (AFO,30) DayNrC*1.0,
     &    0.01*(iGrai+IGird)/OUTPER,
     &    0.01*((iGrai-iNrai)+(iGird-iNird))/OUTPER,
     $    0.01*ievap/OUTPER,
     &    0.0,
     $    0.01*ipeva/OUTPER,
     $    0.01*iptra/OUTPER,
     &    0.01*iruno/OUTPER,
     $    -0.01*gwl,0.01*pond
        elseif (SWAFO.EQ.2) then
          write (AFO,32) DayNrC*1.0,
     &    0.01*iGrai/OUTPER,
     $    0.01*iGird/OUTPER,
     &    0.01*(iGrai-iNrai)/OUTPER,
     $    0.01*(iGird-iNird)/OUTPER,
     $    0.01*ievap/OUTPER,
     &    0.0,
     $    0.01*ipeva/OUTPER,
     $    0.01*iptra/OUTPER,
     &    0.01*iruno/OUTPER,
     $    -0.01*gwl,0.01*pond,0.01*WBALANCE
        endif
        write (AFO,40) (h(node)                 ,node=1,numnod)
        write (AFO,50) (theta(node)             ,node=1,numnod)
        write (AFO,50) (0.01*inqrot(node)/OUTPER,node=1,numnod)
        write (AFO,50) (-0.01*inq(node)/OUTPER  ,node=1,numnod+1)
        if (nrlevs.gt.0) then
          do 300 level=1,nrlevs
            write (AFO,50) (0.01*inqdra(level,node)/OUTPER,
     $        node=1,numnod)
 300      continue
        end if
        If (SWAFO.EQ.2) then
          GC = Min(LAI/3.0d0,1.0d0)
          write (AFO,'(4(1x,f10.4))') GC,LAI,Abs(0.01d0*DRZ),CF
          write (AFO,'(1X,F7.2)') TAV
          write (AFO,4020) (TSoil(node),node = 1,numnod) 
        endif
      end if

30    format (1x,f6.0,7(1x,f7.6),3(1x,f7.4))
32    format (1x,f6.0,9(1x,f7.6),3(1x,f7.4))
40    format (8(1x,e8.3e1))
50    format (10(1x,f8.6))

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTAIR (AIR,AIRNAM,HAIRD,ENVSTR,
     &  YEAR,DAYNR,TCUM,GIRD,CIRR,DayNrC)
C ----------------------------------------------------------------------
C     Date               : 6/6/01          
C     Purpose            : ANIMO/Pestla output: irrigations           
C     Subroutines called : -              
C     Functions called   : -       
C     File usage         : AIRNAM
C ---------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER AIR,YEAR,DAYNR,DayNrC
      integer*4 getun
c     INTEGER(2) iday,imon,iyr

      REAL*8  TCUM,GIRD,CIRR

      CHARACTER AIRNAM*16,IRSTRING*12,ENVSTR*50,FILNAM*80

      LOGICAL HAIRD
C ----------------------------------------------------------------------
      IF (.NOT.HAIRD) THEN

C ---   open output file
        filnam = trim(envstr)//trim(airnam)//'.air'
        air = getun (20,30)
        call fopens(air,filnam,'new','del')

C ---   write header
        WRITE (AIR,'(2A)')'* Filename     : ',FILNAM  
        WRITE (AIR,'(A)') '* FileContent  : irrigations'
        WRITE (AIR,'(A)') '* ProducerId   :     '
c       CALL GETDAT (iyr,imon,iday)
c       WRITE (AIR,'(A,2I3,I5)') '* ProducerDate :',iday,imon,iyr
        WRITE (AIR,'(A)') '* ProducerDate :'
        HAIRD = .TRUE.
      ENDIF

      IRSTRING = '''IRRIGATION'''
      WRITE (AIR,'(A,I5,1X,I3,1X,I5,2F10.3)') IRSTRING,YEAR,
     & DAYNR,DayNrC,GIRD/100,CIRR

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTATE (ATE,ATENAM,HATED,ENVSTR,NUMNOD,
     &  YEAR,DAYNR,TCUM,TEMPI,TSoil,TAV,DayNrC)
C ----------------------------------------------------------------------
C     Date               : 6/6/01          
C     Purpose            : ANIMO output: soil temperatures           
C     Subroutines called : -              
C     Functions called   : -       
C     File usage         : ATENAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi' 

      INTEGER NUMNOD,YEAR,DAYNR,ATE,I,DayNrC
      integer*4 getun

      REAL*8  TCUM,TAV,TEMPI(MACP)
      REAL*8  TSoil(0:MACP)

      CHARACTER ATENAM*16,FILNAM*80
      CHARACTER ENVSTR*50

      LOGICAL HATED

      SAVE

C ----------------------------------------------------------------------
      IF (.NOT.HATED) THEN

C ---   open output file
        filnam = trim(envstr)//trim(atenam)//'.ate'
        ate = getun (20,30)
        call fopens(ate,filnam,'new','del')

C ---   write header
        WRITE (ATE,'(2A)')'* Filename     : ',FILNAM  
        WRITE (ATE,'(A)') '* FileContent  : soil temperature profiles'
        WRITE (ATE,'(A)') '* ProducerId   :     '
        WRITE (ATE,'(A)') '* ProducerDate :'
C ---   write initial temperature profile
        WRITE (ATE,'(I4,1X,I3,1X,I5,100F6.1)') YEAR,DAYNR,DayNrC,
     &         TEMPI(1),(TEMPI(I),I=1,NUMNOD)

        HATED = .TRUE.
      ENDIF

      WRITE (ATE,'(I4,1X,I3,1X,I5,100F6.1)') YEAR,DAYNR,DayNrC,TAV,
     &      (TSoil(I),I=1,NUMNOD)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTAUN (AUN,AUNNAM,HAUND,ENVSTR,
     &  NUMNOD,OUTPER,PERIOD,TCUM,POND,GWL,THETA,H,INQ,
     &  INQROT,NRLEVS,INQDRA,IEVAP,IPEVA,IPTRA,IRUNO,
     &  NUMLAY,BOTCOM,DZ,THETAS,BRUNY,ERUNY,BRUND,ERUND,
     &  LAYER,FLGENU,ALFAMG,THETAR,COFGEN,HTABLE,THETLO,
     &  SWAUN,TEMPI,IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV,TSoil,DayNrC,
     &  DRZ,CF,WBALANCE,prjdes)
C ----------------------------------------------------------------------
C     Date               : 6/6/01           
C     Purpose            : ANIMO output: unformatted hydrological data
C     Subroutines called : 
C     Functions called   : THENODE 
C     File usage         : AUNNAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.fi'

      INTEGER AUN,OUTPER,PERIOD,LAY,LEVEL,NODE,DayNrC
      INTEGER NUMLAY,NUMNOD,NRLEVS,BOTCOM(MAHO),LAYER(MACP)
      INTEGER BRUNY,ERUNY,BRUND,ERUND,THETLO(MAHO),SWAUN
      integer*4 getun,ftype

      REAL*8  GWL,INQDRA(5,MACP),IPREC,IINTC,IEVAP,POND,TCUM,DRZ,CF
      REAL*8  IPEVA,IPTRA,IRUNO,INQ(MACP+1),INQROT(MACP),DZ(MACP)
      REAL*8  TEMPI(MACP),IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV 
      REAL*8  THETA(MACP),H(MACP),THETAS(MACP),THENODE
      REAL*8  ALFAMG(MACP),THETAR(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  DVAL(MAHO),TSoil(0:MACP),WBALANCE

      CHARACTER AUNNAM*16,ENVSTR*50,FILNAM*80, AUNEXT*4
      CHARACTER*80 prjdes

      LOGICAL HAUND,FLGENU(MAHO)
C ----------------------------------------------------------------------
      IF (.NOT.HAUND) THEN 

C ---   open output file
        if (swaun.eq.1) then
          aunext = '.aun'
        elseif (swaun.eq.2) then
          aunext = '.pun'
        endif
        filnam = trim(envstr)//trim(aunnam)//aunext
        aun = getun (20,30)
c        call fopeng(aun,filnam,'new','us',0,'del')
        open(unit=aun,file=filnam,status='unknown',form='unformatted')

c ---   write initial part
        if (swaun.eq.2) then
          ftype = 2
          call writehead(aun,ftype,prjdes)
        endif
        if (period.eq.0) period = -10
        write (AUN) BRUNY,ERUNY, 
     &  float(BRUND-1),float(ERUND),float(PERIOD)
        if (period.eq.-10) period = 0
        write (AUN) numnod,numlay,nrlevs
        write (AUN) (botcom(lay),lay=1,numlay)
        write (AUN) (real(thetaS(botcom(lay))), lay=1,numlay)

        DO 4 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-100.d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
4       CONTINUE
        write (AUN) (REAL(DVAL(lay)),lay=1,numlay) 

        DO 6 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-15849.0d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
6       CONTINUE

        write (AUN) (REAL(DVAL(lay)),lay=1,numlay) 

        write (AUN) (0.01*real(dz(node)),node=1,numnod)
        write (AUN) (real(theta(node)),node=1,numnod)
        write (AUN) -0.01*real(gwl),0.01*real(pond)
        If (SWAUN.EQ.2) then
          write (AUN) (real(tempi(node)),node = 1,numnod) 
        endif
        HAUND = .TRUE.
        RETURN
      ENDIF

C --- unformatted output of data - dynamic part
      if (HAUND) then
        if (SWAUN.eq.1) then
          write (AUN) real(DayNrC),
     &    0.01*real(iGrai+IGird)/OUTPER,
     &    0.01*(real(iGrai-iNrai)+real(iGird-iNird))/OUTPER,
     $    0.01*real(ievap)/OUTPER,
     &    0.0,
     $    0.01*real(ipeva)/OUTPER,
     $    0.01*real(iptra)/OUTPER,
     &    0.01*real(iruno)/OUTPER,
     $    -0.01*real(gwl),0.01*real(pond)
        elseif (SWAUN.EQ.2) then
          write (AUN) real(DayNrC),
     &    0.01*real(iGrai)/OUTPER,
     $    0.01*real(iGird)/OUTPER,
     &    0.01*real(iGrai-iNrai)/OUTPER,
     $    0.01*real(iGird-iNird)/OUTPER,
     $    0.01*real(ievap)/OUTPER,
     &    0.0,
     $    0.01*real(ipeva)/OUTPER,
     $    0.01*real(iptra)/OUTPER,
     &    0.01*real(iruno)/OUTPER,
     $    -0.01*real(gwl),0.01*real(pond),0.01*real(WBALANCE)
        endif
        write (AUN) (real(h(node))           ,node=1,numnod)
        write (AUN) (real(theta(node))       ,node=1,numnod)
        write (AUN) (0.01*real(inqrot(node))/OUTPER,node=1,numnod)
        write (AUN) (-0.01*real(inq(node))/OUTPER  ,node=1,numnod+1)
        if (nrlevs.gt.0) then
          do 310 level=1,nrlevs
            write (AUN) (0.01*real(inqdra(level,node))/OUTPER,
     $        node=1,numnod)
 310      continue
        end if
        If (SWAUN.EQ.2) then
          GC = Min((LAI/3.0d0),1.0d0)
          write (AUN) real(GC),real(LAI),abs(0.01*real(DRZ)),real(CF)
          write (AUN) real(TAV)
          write (AUN) (real(TSoil(node)),node = 1,numnod) 
        endif
      end if

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTBAL (SW1FIL,OUTNAM,ENVSTR,DAYNR,YEAR,BRUND,BRUNY,
     &  BAYRD,BAYRY,CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,CQROT,CEVAP,CQBOT,
     &  CQDRAIN,CQDRA,CQDRAR,VOLACT,VOLINI,Z,DZ,SWDRA,NUMNOD,
     &  NRLEVS,SAMINI,SAMPRO,SAMCRA,SQPREC,SQIRRIG,SQBOT,DECTOT,ROTTOT,
     &  SQRAP,SQDRA,SWSOLU,POND,runnr)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write overview balances to OUTNAM.BAL file
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.BAL
C ---------------------------------------------------------------------
      IMPLICIT NONE
      include 'Param.Fi'

      INTEGER   DAYNR,YEAR,BRUND,BRUNY,BAYRD,BAYRY,BAL,SWDRA
      INTEGER   DSTART,MSTART,YSTART,DEND,MEND,YEND,NUMNOD,NRLEVS,I
      INTEGER   SWSOLU,SW1FIL,runnr
      integer*4 getun

      REAL*8    CEVAP,CGIRD,CGRAI,CNIRD,CNRAI,CQBOT,INTERC
      REAL*8    CQROT,CRUNO,CQDRAR,CQDRA,CQDRAIN(5),DZ(MACP),Z(MACP)
      REAL*8    SAMINI,SAMPRO,SAMCRA,SQPREC,SQIRRIG,SQBOT,DECTOT,ROTTOT
      REAL*8    SQRAP,SQDRA,POND
      REAL*8    VOLACT,VOLINI

      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50
C ----------------------------------------------------------------------

C --- open output file
      if (sw1fil.eq.0 .or. 
     &    (sw1fil.eq.1.and.runnr.eq.1)) then
        filnam = trim(envstr)//trim(outnam)//'.bal'
        bal = getun (20,30)
        call fopens(bal,filnam,'new','del')
      endif

C --- start date of sub-run
      IF (BAYRY.GT.BRUNY.OR.
     &  ((BAYRY.EQ.BRUNY).AND.(BAYRD.GT.BRUND))) THEN
        CALL NRTODA (BAYRY,BAYRD,MSTART,DSTART)
        YSTART = BAYRY
      ELSE
        CALL NRTODA (BRUNY,BRUND,MSTART,DSTART)
        YSTART = BRUNY
      ENDIF

C --- end date of sub-run
      IF (DAYNR.GT.1) THEN
        CALL NRTODA (YEAR,(DAYNR-1),MEND,DEND)
          YEND = YEAR
      ELSE
          MEND = 12
          DEND = 31
          YEND = YEAR - 1
      ENDIF

C --- write output record

      WRITE (BAL,20) DSTART,MSTART,YSTART,DEND,MEND,YEND
      WRITE (BAL,22) (-Z(NUMNOD) + 0.5*DZ(NUMNOD))

      IF (SWSOLU .EQ. 1) THEN
          WRITE (BAL,24) (VOLACT+POND),(SAMPRO+SAMCRA),VOLINI,SAMINI,
     &    (VOLACT+POND-VOLINI),(SAMPRO+SAMCRA-SAMINI)
      ELSE
          WRITE (BAL,25) (VOLACT+POND),VOLINI,(VOLACT+POND-VOLINI)
      ENDIF

      INTERC = CGRAI+CGIRD-CNRAI-CNIRD
      WRITE (BAL,26) CGRAI,INTERC,CGIRD,CRUNO,CQBOT,CQROT,CEVAP,CQDRAR

      IF (SWDRA .NE. 0) THEN
        DO 100 I = 1,NRLEVS
          WRITE (BAL,28) I,CQDRAIN(I)
 100    CONTINUE
      ENDIF
      WRITE (BAL,30) (CGRAI+CGIRD+CQBOT),(INTERC+CRUNO+CQROT+CEVAP+
     &   CQDRAR+CQDRA)

      IF (SWSOLU .EQ. 1) THEN
        WRITE (BAL,34) SQPREC,DECTOT,SQIRRIG,ROTTOT,SQBOT,SQRAP,SQDRA
        WRITE (BAL,36) (SQPREC+SQIRRIG+SQBOT),
     &    (DECTOT+ROTTOT+SQRAP+SQDRA)
      ENDIF

      if (sw1fil.eq.0)  CLOSE (BAL)

 20   FORMAT('Period',t20,':',t22,i2,'/',i2.2,'/',i4,
     &   ' until  ',i2,'/',i2.2,'/',i4)
 22   FORMAT('Depth soil profile',t20,':',f8.2,' cm')
 24   FORMAT(/,T13,'Water storage',t36,'Solute storage',/,
     &   'Final',t9,':',t15,f8.2,' cm',t31,e12.4,' mg/cm2',/,
     &   'Initial',t9,':',t15,f8.2,' cm',t31,e12.4,' mg/cm2',/,
     &    t13,13('='),t33,17('='),/,
     &   'Change',t15,f8.2,' cm',t31,e12.4,' mg/cm2')
 25   FORMAT(/,T13,'Water storage',/,'Final',t9,':',t15,f8.2,' cm',/,
     &   'Initial',t9,':',t15,f8.2,' cm',/,
     &    t13,13('='),/,'Change',t15,f8.2,' cm')
 26   FORMAT(//,'Water balance components (cm)',//,
     &   'In',t30,'Out',/,25('='),t30,28('='),/,
     &   'Rain',t16,':',f9.2,t30,'Interception',t48,':',f9.2,/,
     &   'Irrigation',t16,':',f9.2,t30,'Runoff',t48,':',f9.2,/,
     &   'Bottom flux',t16,':',f9.2,t30,'Transpiration',t48,':',f9.2,
     &   /,t30,'Soil evaporation',t48,':',f9.2,/,
     &   t30,'Crack flux',t48,':',f9.2)
 28   FORMAT(t30,'Drainage level',i2,t48,':',f9.2)
 30   FORMAT(25('='),t30,28('='),/,
     &   'Sum',t16,':',f9.2,t30,'Sum',t48,':',f9.2)
 34   FORMAT(//,'Solute balance components (mg/cm2)',//,
     &   'In',t30,'Out',/,25('='),t30,28('='),/,
     &   'Rain',t13,':',e12.4,t30,'Decomposition',t45,':',e12.4,/,
     &   'Irrigation',t13,':',e12.4,t30,'Root uptake',t45,':',e12.4,/,
     &   'Bottom flux',t13,':',e12.4,t30,'Cracks',t45,':',e12.4,/,
     &   t30,'Drainage',t45,':',e12.4)
 36   FORMAT(25('='),t30,28('='),/,
     &   'Sum',t13,':',e12.4,t30,'Sum',t45,':',e12.4,/)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTINC (OUTNAM,INC,HINCD,ENVSTR,DAYNR,YEAR,
     &  IGRAI,IGIRD,IINTC,IRUNO,IPTRA,IQROT,IPEVA,IEVAP,IQDRA,
     &  IQDRAR,IQBOT,T)
C ----------------------------------------------------------------------
C     Date               : 8/04/99
C     Purpose            : write water balance increments to OUTNAM.WBA file
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.INC
C ---------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   INC,DAYNR,YEAR,D,M
      integer*4 getun

      REAL*8    IGRAI,IGIRD,IINTC,IRUNO,IPTRA,IQROT,IPEVA,IEVAP,IQDRA
      REAL*8    IQDRAR,IQBOT,T

      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50

      LOGICAL   HINCD
C ----------------------------------------------------------------------
      IF (.NOT.HINCD) THEN

C ---   open output file
        filnam = trim(envstr)//trim(outnam)//'.inc'
        inc = getun (20,30)
        call fopens(inc,filnam,'new','del')

C --- write header
        WRITE (INC,10)
 10     FORMAT ('* Water balance increments (cm/period)',//,
     &'*      Date Day    Rain   Irrig  Interc  Runoff   Transpiration',
     &'     Evaporation Drainage QBottom',/,
     &'*dd/mm/yyyy  nr   gross   gross                     pot     act',
     &'     pot     act      net     net',/,
     &'*<========><==><======><======><======><======><======><======>',
     &'<======><======><=======><======>')

        HINCD = .TRUE.
      ENDIF

C --- Write output .INC file

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record
      WRITE (INC,20) D,M,YEAR,NINT(T),IGRAI,IGIRD,IINTC,IRUNO,
     &  IPTRA,IQROT,IPEVA,IEVAP,(IQDRAR+IQDRA),IQBOT
20    FORMAT (I3,'/',I2.2,'/',2I4,8F8.3,F9.3,F8.3)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTSBA (OUTNAM,SBA,HSBAD,ENVSTR,DAYNR,YEAR,
     &   DECTOT,ROTTOT,SAMPRO,SAMCRA,SQBOT,SQSUR,SQRAP,SAMAQ,
     &   SQDRA,T,SOLBAL,SQPREC,SQIRRIG)
C ----------------------------------------------------------------------
C     Date               : 18/08/98                            
C     Purpose            : output of salt balance
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.SBA
C ---------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      INTEGER   SBA,DAYNR,YEAR
      integer*4 getun
      REAL*8    SAMPRO,SAMCRA,SQBOT,SQSUR,SQRAP,SAMAQ,SQDRA,T
      REAL*8    SOLBAL,DECTOT,ROTTOT,SQPREC,SQIRRIG

      CHARACTER OUTNAM*16,ENVSTR*50

      LOGICAL   HSBAD
C ----------------------------------------------------------------------
C --- local
      INTEGER   D,M
      CHARACTER filnam*80
C ----------------------------------------------------------------------
      IF (.NOT.HSBAD) THEN 

C ---   open output file
        filnam = trim(envstr)//trim(outnam)//'.sba'
        sba = getun (20,30)
        call fopens(sba,filnam,'new','del')

C --- write headers
        WRITE (SBA,10)
10    FORMAT (
     &'*DATE       DAY      SQTOP      DECTOT      ROTTOT       SAMPRO',
     &'      SAMCRA       SQBOT       SQDRA       SQRAP       SAMAQ',
     &'       SQSUR     SOLBAL      DATE*',/,
     &'*dd/mm/yyyy  nr      mg/cm2      mg/cm2      mg/cm2      mg/cm2',
     &'      mg/cm2      mg/cm2      mg/cm2      mg/cm2      mg/cm2',
     &'      mg/cm2     mg/cm2 dd/mm/yyyy',/,
     &'*<========><==><==========><==========><==========><==========>',
     &'<==========><==========><==========><==========><==========>',
     &'<==========><=========><=========>')

        HSBAD = .TRUE.
      ENDIF

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- Write output record .SBA file

      WRITE(SBA,15) D,M,YEAR,NINT(T),(SQPREC+SQIRRIG),DECTOT,ROTTOT,
     &  SAMPRO,SAMCRA,SQBOT,SQDRA,SQRAP,SAMAQ,SQSUR,SOLBAL,D,M,YEAR
 15   FORMAT(I3,'/',I2.2,'/',2I4,10E12.4,1E11.2,I3,'/',I2.2,'/',I4)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTTEP (OUTNAM,TEP,HTEPD,ENVSTR,DAYNR,YEAR,
     &  NUMNOD,TSoil,T)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write soil temperatures to output file
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.TEP
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER   TEP,DAYNR,YEAR,NODE,NUMNOD,D,M
      integer*4 getun
      REAL*8    T, TSoil(0:MACP)

      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50

      LOGICAL   HTEPD
C ----------------------------------------------------------------------
      IF (.NOT.HTEPD) THEN

C ---   open output file
        filnam = trim(envstr)//trim(outnam)//'.tep'
        tep = getun (20,30)
        call fopens(tep,filnam,'new','del')

C ---   write header
        WRITE (TEP,10)
10      FORMAT (
     &'*DATE      DAY Compartment number------------------------------'
     & ,/,
     &'*dd/mm/yyyy nr      1     6    11    16    21    26    31    36'
     & ,/,
     &'*<========><==><====><====><====><====><====><====><====><====>')
        HTEPD = .TRUE.
      ENDIF
 
C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

      write (TEP,1010) D,M,YEAR,NINT(T),(TSoil(node),node=1,numnod,5)
1010  format(I3,'/',I2.2,'/',2I4,20F6.1)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTVAP (SWSOLU,OUTNAM,VAP,HVAPD,ENVSTR,DAYNR,
     &  YEAR,NUMNOD,Z,THETA,H,CML,SWTEMP,TSoil,T,Q)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write to additional short output file
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.VAP
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER   SWSOLU,VAP,DAYNR,YEAR,NUMNOD,SWTEMP

      REAL*8    Z(MACP),CML(MACP),T
      REAL*8    THETA(MACP),H(MACP),TSoil(0:MACP),Q(MACP+1)

      CHARACTER OUTNAM*16,ENVSTR*50

      LOGICAL   HVAPD
C ----------------------------------------------------------------------
C --- local
      INTEGER   d,m,node
      integer*4 getun

      CHARACTER filnam*80
      REAL*8 WFLUX,SFLUX
C ----------------------------------------------------------------------
      IF (.NOT.HVAPD) THEN 

C ---   open output file
        filnam = trim(envstr)//trim(outnam)//'.vap'
        vap = getun (20,30)
        call fopens(vap,filnam,'new','del')

C --- write header
        WRITE (VAP,50) 

 50   FORMAT ('*      date       depth  water       head     solute  ',
     &        'temp       Wflux       Sflux',/,
     &        '*                    cm    (-)         cm     mg/cm3  ',
     &        '   C        cm/d    mg/cm2/d')


c        HVAPD = .TRUE.

      ENDIF

C --- Write output .VAP file

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record

      DO 20 NODE = 1,NUMNOD
        if (SWTEMP.EQ.0) TSoil(NODE) = -99.9d0
        if (SWSOLU.EQ.0) CML(NODE)  = 0.0
        WFLUX = (Q(NODE)+Q(Node+1))/2
        SFLUX = WFLUX*CML(NODE)

        IF  (.not. HVAPD) then
          WRITE (VAP,79) D,M,YEAR,NINT(T),Z(NODE),THETA(NODE),
     &                   H(NODE),CML(NODE), TSoil(NODE),WFLUX,SFLUX
        ELSE
          WRITE (VAP,80) D,M,YEAR,NINT(T),Z(NODE),THETA(NODE),
     &                   H(NODE),CML(NODE), TSoil(NODE),WFLUX,SFLUX
        ENDIF

20    CONTINUE
      HVAPD = .true.

      WRITE (VAP,'(2X)')

 79   FORMAT('*',I2,'/',I2.2,'/',2I4,1X,F7.1,F7.3,F11.3,E11.3,F6.1,
     & 2E12.3)
 80   FORMAT(    I3,'/',I2.2,'/',2I4,1X,F7.1,F7.3,F11.3,E11.3,F6.1,
     & 2E12.3)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTWBA (SW1FIL,OUTNAM,WBA,HWBAD,ENVSTR,DAYNR,YEAR,
     &  CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,CQROT,CEVAP,CQBOT,CQDRA,
     &  CPTRA,CPEVA,GWL,T,CQDRAR,VOLACT,VOLINI,POND,RUNNR,
     &  SWSCRE,TCUM,DayNrC,WBALANCE)
C ----------------------------------------------------------------------
C     Date               : 6/6/01
C     Purpose            : write water balance data to OUTNAM.WBA file
C     Subroutines called : ,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.WBA
C ---------------------------------------------------------------------
      IMPLICIT NONE

c -   Global
      INTEGER   WBA,DAYNR,DAYNRC,YEAR,D,M,RUNNR,SWSCRE, sw1fil
      integer*4 getun
      REAL*8    CEVAP,CGIRD,CGRAI,CNIRD,CNRAI,CPEVA,CPTRA,CQBOT,DSTOR
      REAL*8    CQDRA,CQROT,CRUNO,GWL,T,CQDRAR,POND,TCUM
      REAL*8    VOLACT,VOLINI,WBALANCE
      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50
      LOGICAL   HWBAD
c -   Local
      INTEGER   DaynrDum

C ----------------------------------------------------------------------
      IF (.NOT.HWBAD) THEN

C ---   open output file
        filnam = trim(envstr)//trim(outnam)//'.wba'
        wba = getun (20,30)
        call fopens(wba,filnam,'new','del')

C --- write header WBA file
        WRITE (WBA,10)
10      FORMAT (
     &   '*    DATE     DAY    RAIN [cm]     IRR [cm]     RUO    TRA ',
     &   '[cm]      EVS [cm]      FLUX [cm]      DSTOR    GWL   WBAL ',
     &   '    Date',/,
     &   '*dd/mm/yyyy    nr   gro....net    gro...net      cm   pot..',
     &   '..act    pot....act    lat.....bot      cm      cm      cm ',
     &   ' dd/mm/yyyy',/,
     &   '*<========><=====><=====><=====><====><====><======><=====>',
     &   '<=====><=====><=====><======><======><======><======><====>',
     &   '<==========>')

C --- write header screen output
        IF (SWSCRE.EQ.1) THEN
          WRITE (*,20) RUNNR
 20       FORMAT(/,'   RUN ',I2,T20,
     &           '  Cumulative water balance components (cm)',/)
          WRITE (*,22)
 22       FORMAT ('         Date   Rain  Irrig Runoff Transp Evapor',
     &            '  Drain   Qbot    Gwl   WBal'/,
     &            '               gross  gross        actual actual',
     &            '    net    net           cum')
        endif
        HWBAD = .TRUE.
      ENDIF
C --- write output water balance components

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record WBA file
      DSTOR = REAL(VOLACT) + POND - REAL(VOLINI)
      if (sw1fil.eq.1) then
        DaynrDum = DaynrC
      else
        DaynrDum = nint(T)
      endif
      WRITE (WBA,30) D,M,YEAR,DaynrDum,CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,
     &  CPTRA,CQROT,CPEVA,CEVAP,(CQDRA+CQDRAR),CQBOT,DSTOR,GWL,
     &  WBALANCE,D,M,YEAR
 30   FORMAT (1x,I2.2,'/',I2.2,'/',i4,I6,2(1x,F6.2),2(1x,F5.1),1x,F7.2,
     &      4(1x,F6.2),3(1x,F7.2),1x,F7.1,1x,F5.2,1x,I3,'/',I2.2,'/',I4)

C --- write output record screen
      IF (SWSCRE.EQ.1) THEN
        WRITE (*,40) D,M,YEAR,CGRAI,CGIRD,CRUNO,CQROT,CEVAP,
     &               (CQDRA+CQDRAR),CQBOT,GWL,WBALANCE
 40     FORMAT(2x,I3,'/',I2.2,'/',I4,7F7.2,F7.1,F7.2)
      elseif (SWSCRE.EQ.2) then
        Write(*,'(1H+,4x,i4,2X,i3,1X,I6)')  year,daynr,DayNrC
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE PENMON (DAYNR,LAT,ALT,A,B,RCS,RCC,RAD,TAV,HUM,
     & WIN,RSC,ES0,ET0,EW0,SWCF,CF,ISTAGE)
C ----------------------------------------------------------------------
C     Date               : 14/01/99
C     Purpose            : Calculation of potential evaporation & 
C                          transpiration rates from a bare soil surface,
C                          a dry and a wet crop canopy;           
C                          based on the Penman - Monteith approach.
C
C     Formal parameters  : (I = input, O = output)
C     DAYNR  daynumber (January 1st = 1) [-]...........................I
C     LAT    latitude [degr.,dec.degr., N=+,S=-].......................I
C     ALT    altitude above mean sea level [m].........................I
C     A      first  Angstrom coefficient [-]...........................I
C     B      second Angstrom coefficient [-]...........................I
C     RCS    reflection coefficient soil [-]...........................I
C     RCC    reflection coefficient crop [-]...........................I
C     RAD    incoming short wave radiation [J/m2/d]....................I
C     TAV    average day temperature [C]...............................I
C     HUM    vapour pressure [kPa].....................................I
C     WIN    windspeed at 2 m height [m/s].............................I
C     RCS    minimum canopy resistance [s/m]...........................I
C     ES0    potential evaporation rate from a  bare soil [mm/d].......O
C     ET0    potential transpiration rate from a dry crop [mm/d].......O
C     EW0    potential transpiration rate from a wet crop [mm/d].......O
C     SWCF   CF contains crop factor (=1) or crop height (=2)
C     CF     crop factor [-] or crop height [cm]
C     IStage no crop [=1] or yes crop [=2]
C
C     Subroutines called : ASTRO
C
C     Functions called   : LIMITr8
C
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER DAYNR,SWCF,IStage

      REAL*8  LIMITR8,LAMBDA,LAT,CHRef,CF
      REAL*8  A,ALT,ATMTR,B,COSLD,CP,DAYL,DAYLP,DELTA,DSINB,DSO,EA
      REAL*8  ED,ES0,ET0,ETAERC,ETAERS,ETAERW,ETRADC,ETRADS,ETRADW,EW0
      REAL*8  G,GAMMA,GAMMOC,GAMMOS,GAMMOW,HUM,PALT,PI,RAC,RAD,RAS,RAW
      REAL*8  RCC,RCS,RELSSD,RHO,RNC,RNL,RNS,RNW,RSC,RSS,RSW,SC,SINLD
      REAL*8  TAV,TAVK,TKV,UD,VPD,WIN,CH

      PARAMETER (CHRef = 12.0d0, PI = 3.1415962d0)
C ----------------------------------------------------------------------
C --- avoid zero crop height and crop height > 3.0 m
      If (Istage.EQ.1) then
        CH = CHref
      else 
        If (SWCF.EQ.1) Then
          CH = CHRef
        Else
          CH = MAX (CF,0.1d0)
          CH = Min (CF,295.0d0)
        Endif
      Endif
     
C --- conversion of average temperature from [C] to [K]
      TAVK = TAV+273.0d0

C --- atmospheric pressure at elevation ALT [kPa]
      PALT = 101.3*((TAVK-0.0065*ALT)/TAVK)**5.26

C --- latent heat of vaporization [MJ/kg]
      LAMBDA = 2.501d0-0.002361*TAV

C --- saturation vapour pressure at TAV [kPa]
      EA  = 0.611*EXP(17.27*TAV/(TAV+237.3d0))
C --- measured vapour pressure not to exceed saturated vapour pressure
      ED  = MIN(HUM,EA)
C --- vapour pressure deficit [kPa]
      VPD = EA-ED

C --- slope vapour pressure curve [kPa/C]
      DELTA = 4098*EA/(TAV+237.3d0)**2

C --- psychrometric constant [kPa/C]
      GAMMA = 0.00163*PALT/LAMBDA

C --- atmospheric density [kg/m3]
      TKV = TAVK/(1.0d0-0.378*ED/PALT)
      RHO = 3.486*PALT/TKV

C --- specific heat moist air [kJ/kg/C]
      CP = 622*GAMMA*LAMBDA/PALT

C --- day wind [m/s], avoid zero windspeed
      UD = MAX (WIN*1.33,0.0001d0)

C --- surface resistance [s/m] - Soil & Wet crop
      RSS = 0
      RSW = 0

C --- aerodynamic resistance [s/m] - Soil, Crop & Wet crop
      RAS = LOG((200.0d0-0.67*0.1d0)/0.123/0.1)*LOG((200.0d0-0.67*0.1d0)
     &      /0.0123/0.1)/(0.1681*UD)
      RAC = LOG((200.0d0-0.67*CH)/0.123/CH)*LOG((200.0d0-0.67*CH)/
     &      0.0123/CH)/(0.1681*UD)
      RAW = LOG((200.0d0-0.67*CH)/0.123/CH)*LOG((200.0d0-0.67*CH)/
     &      0.0123/CH)/(0.1681*UD)

C --- modified psychrometric constant [kPa/C] - Soil, Crop & Wet crop 
      GAMMOS = GAMMA*(1.0d0+RSS/RAS)
      GAMMOC = GAMMA*(1.0d0+RSC/RAC)
      GAMMOW = GAMMA*(1.0d0+RSW/RAW)

C --- net short wave radiation [MJ/m**2/d] - Soil, Crop & Wet crop
      RNS = (1.0d0-RCS)*RAD/1000000
      RNC = (1.0d0-RCC)*RAD/1000000
      RNW = (1.0d0-RCC)*RAD/1000000

C --- net long wave radiation [MJ/m**2/d]
      CALL ASTRO (DAYNR,LAT,DAYL,DAYLP,SINLD,COSLD)
      DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*SQRT(1.0d0-
     &         (SINLD/COSLD)**2)/PI)
      SC     = 1370.*(1.0d0+0.033*COS(2.*PI*DAYNR/365.))
      DSO    = SC*DSINB
      ATMTR  = RAD/DSO
      RELSSD = LIMITR8(0.0d0,1.0d0,(ATMTR-A)/B)
      RNL    = 4.9d-9*TAVK**4*(0.34d0-0.14*SQRT(ED))*(0.1d0+0.9*RELSSD)

C --- soil heat flux [MJ/m2/d]
      G = 0.0

C --- aerodynamic term of the PM equation [mm/d] - Soil, Crop & Wet crop
      ETAERS = (86.4/LAMBDA)*(1/(DELTA+GAMMOS))*(RHO*CP*VPD/RAS)
      ETAERC = (86.4/LAMBDA)*(1/(DELTA+GAMMOC))*(RHO*CP*VPD/RAC)
      ETAERW = (86.4/LAMBDA)*(1/(DELTA+GAMMOW))*(RHO*CP*VPD/RAW)

C --- radiation term of the PM equation [mm/d] - Soil, Crop & Wet crop
      ETRADS = DELTA/(DELTA+GAMMOS)*(RNS-RNL-G)*1/LAMBDA      
      ETRADC = DELTA/(DELTA+GAMMOC)*(RNC-RNL-G)*1/LAMBDA      
      ETRADW = DELTA/(DELTA+GAMMOW)*(RNW-RNL-G)*1/LAMBDA      

C --- sum of both terms [mm/d] - Soil, Crop and Wet crop      
      ES0 = MAX (0.0d0,ETAERS+ETRADS)
      ET0 = MAX (0.0d0,ETAERC+ETRADC)
      EW0 = MAX (0.0d0,ETAERW+ETRADW)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RADIAT (IDAY,HOUR,DAYL,SINLD,COSLD,AVRAD,SINB,
     $                   PARDIR,PARDIF)
C ----------------------------------------------------------------------
C --- Author: Daniel van Kraalingen, 1986
C --- Calculates the fluxes of diffuse and direct photosynthetically
C --- active radiation from the total daily shortwave radiation actually
C --- received (AVRAD) for a given day of the year and hour of the day.
C --- The input variables DAYL, SINLD and COSLD are calculated in ASTRO.
C --- For more information: see Spitters et al. (1988).
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER IDAY

      REAL*8  AOB,ATMTR,AVRAD,COSLD,DAYL,DSINB,DSINBE,DSO,FRDIF,HOUR
      REAL*8  PAR,PARDIF,PARDIR,PI,SC,SINB,SINLD

      DATA    PI /3.1415926d0/
C ----------------------------------------------------------------------
C --- calculations on solar elevation
C --- sine of solar elevation SINB
      AOB    = SINLD/COSLD
      SINB   = MAX (0.0d0,SINLD+COSLD*COS(2.0*PI*(HOUR+12.0d0)/24.0))
C --- integral of SINB
      DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*SQRT(1.0d0-AOB*AOB)/PI)
C --- integral of SINB, corrected for lower atmospheric transmission
C --- at low solar elevations
      DSINBE = 3600.*(DAYL*(SINLD+0.4*(SINLD*SINLD+COSLD*COSLD*0.5))+
     $         12.0*COSLD*(2.0d0+3.0*0.4*SINLD)*SQRT(1.0d0-AOB*AOB)/PI)

C --- solar constant and daily extraterrestrial radiation
      SC     = 1370.*(1.0d0+0.033*COS(2.0*PI*IDAY/365.))
      DSO    = SC*DSINB

C --- diffuse light fraction from atmospheric transmission
      ATMTR  = AVRAD/DSO
      IF (ATMTR.GT.0.75d0) FRDIF = 0.23d0
      IF (ATMTR.LE.0.75d0.AND.ATMTR.GT.0.35d0) FRDIF = 1.33d0-1.46*ATMTR
      IF (ATMTR.LE.0.35d0.AND.ATMTR.GT.0.07d0) 
     $ FRDIF = 1.0d0-2.3*(ATMTR-0.07d0)**2
      IF (ATMTR.LE.0.07d0) FRDIF = 1.0d0

C --- photosynthetic active radiation, diffuse and direct
      PAR    = 0.5*AVRAD*SINB*(1.0d0+0.4*SINB)/DSINBE
      PARDIF = MIN (PAR,SINB*FRDIF*ATMTR*0.5*SC)
      PARDIR = PAR-PARDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDBBC (runnr,iseq,sw1fil,BBCFIL,ENVSTR,BAYRD,BAYRY,
     & GWLTAB,SHAPE,RIMLAY,AQAVE,AQAMP,AQTAMX,AQOMEG,QBOTAB,COFQHA,
     & COFQHB,HGITAB,SWBOTB,LOGF,YEAR,FMAY,HDRAIN,AQATAB,RECIRES)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                               
C     Purpose            : reading b.c.'s at bottom of soil profile,   
C                          for the entire calculation period.          
C     Subroutines called : DATONR,           
C     Functions called   : FROMTO                                      
C     File usage         : .BBC, file containing bottom boundary data  
C ----------------------------------------------------------------------
      IMPLICIT NONE
      include 'param.fi'

      INTEGER   BBC,FROMTO,BAYRD,BAYRY,SWBOTB,NR,LOGF,YEAR,FMAY
      INTEGER   runnr,iseq,sw1fil
      REAL*8    GWLTAB(732),QBOTAB(732),HGITAB(732)
      REAL*8    SHAPE,RIMLAY,AQAVE,AQAMP,AQTAMX,AQOMEG,HDRAIN
      REAL*8    AQPER,COFQHA,COFQHB,PI,RECIRES,AQATAB(732)

      CHARACTER BBCFIL*16,ENVSTR*50,FILNAM*80

      PARAMETER (PI=3.14159d0)
C ----------------------------------------------------------------------
C --- local
      integer   swc2,dd(366),mm(366),yy(366),i,ifnd, nday
      integer*4 getun2, datea(6)
      real*8    date_bbc1(mabbc),gwlevel(mabbc)
      real*8    date_bbc2(mabbc),qbot(mabbc)
      real*8    date_bbc5(mabbc),PressHead(mabbc)
      real*8    date_bbc9a(mabbc),date_bbc9b(mabbc),HeadAqave(mabbc)
      real*8    c2ave,c2amp,c2max,value(366)
      real*4    fsec

      save

C ----------------------------------------------------------------------

c --- Initialise
      do i = 1,732
        aqatab(i)   = 0.0D0
        gwltab(i)   = 0.0D0
        hgitab(i)   = 0.0D0
        qbotab(i)   = 0.0D0
      end do

C --- open file
      filnam = trim(envstr)//trim(bbcfil)//'.bbc'
      bbc = getun2 (10,19,2)
      call rdinit(bbc,logf,filnam)

c --- option for bottom boundary condition
      call rdsinr ('swbotb',1,9,swbotb)

C --- given groundwaterlevel
      if (swbotb.eq.1) then

C -     data from files for each subrun/year
        if (sw1fil.eq.0) then
          call rdainr ('dd_bbc1',1,31,dd,366,ifnd)
          call rdfinr ('mm_bbc1',1,12,mm,366,ifnd)
          call rdfdor ('gwlevel',-1000.0d0,10.0d0,gwlevel,mabbc,ifnd)
          nday = ifnd

C -     data from one file
        else if (sw1fil.eq.1) then
c         first year only
          if (runnr.eq.1 .or. iseq.eq.0) then
            call rdatim ('date_bbc1',date_bbc1,mabbc,ifnd)
            call rdfdor ('gwlevel',-1000.0d0,10.0d0,gwlevel,mabbc,ifnd)
          end if
C -       read data from arrays
          nday = 0
          do i = 1,ifnd
            call dtdpar(date_bbc1(i),datea,fsec)
            if ((datea(1).eq.bayry .and. datea(2).ge.fmay) .or. 
     &          (datea(1).eq.(bayry+1) .and. datea(2).lt.fmay)) then
              nday = nday + 1
              yy(nday) = datea(1)
              mm(nday) = datea(2)
              dd(nday) = datea(3)
              value(nday) = gwlevel(i)
            end if
          end do
        end if

c -     store values in gwltab
        do i = 1,nday
          if (mm(i).ge.fmay) yy(i) = bayry
          if (mm(i).lt.fmay) yy(i) = bayry+1
        end do
        do i = 1,nday
          call datonr (yy(i),mm(i),dd(i),nr)
          gwltab(i*2)   = value(i) 
          gwltab(i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
        enddo


C --- regional bottom flux is given                             
      else if (swbotb.eq.2) then
        call rdsinr ('swc2',1,2,swc2)
        if (swc2 .eq. 1) then
          call rdsdor ('c2ave',-10.0d0,10.0d0,c2ave)
          call rdsdor ('c2amp',-10.0d0,10.0d0,c2amp)
          call rdsdor ('c2max',1.0d0,366.0d0,c2max)
          c2max = fromto (bayrd,bayry,int(c2max),year)+1
c ---     fill table with sine function
          do 19 i = 1,366
            qbotab(i*2)   = c2ave+c2amp*cos(0.0172*(i-c2max)) 
            qbotab(i*2-1) = i
 19       continue 
        else
C -     data from files for each subrun/year
          if (sw1fil.eq.0) then
            call rdainr ('dd_bbc2',1,31,dd,366,ifnd)
            call rdfinr ('mm_bbc2',1,12,mm,366,ifnd)
            call rdfdor ('qbot',-1000.0d0,10.0d0,qbot,mabbc,ifnd)
            nday = ifnd
C -       data from one file
          else if (sw1fil.eq.1) then
c           first year only
            if (runnr.eq.1 .or. iseq.eq.0) then
              call rdatim ('date_bbc2',date_bbc2,mabbc,ifnd)
              call rdfdor ('qbot',-1000.0d0,10.0d0,qbot,mabbc,ifnd)
            end if
C -         read data from arrays
            nday = 0
            do i = 1,ifnd
              call dtdpar(date_bbc2(i),datea,fsec)
              if ((datea(1).eq.bayry .and. datea(2).ge.fmay) .or. 
     &          (datea(1).eq.(bayry+1) .and. datea(2).lt.fmay)) then
                nday = nday + 1
                yy(nday) = datea(1)
                mm(nday) = datea(2)
                dd(nday) = datea(3)
                value(nday) = qbot(i)
              end if
            end do
          end if
c -       store values in gbotab
          do i = 1,nday
            if (mm(i).ge.fmay) yy(i) = bayry
            if (mm(i).lt.fmay) yy(i) = bayry+1
          end do
          do i = 1,nday
            call datonr (yy(i),mm(i),dd(i),nr)
            qbotab(i*2)   = value(i) 
            qbotab(i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
          enddo
        end if

C --- calculated flux through the bottom of the profile
      else if (swbotb.eq.3) then
        call rdsdor ('shape',0.0d0,1.0d0,shape)
        call rdsdor ('hdrain',-1.0d4,0.0d0,hdrain)
        call rdsdor ('rimlay',0.0d0,1.0d5,rimlay)
        call rdsdor ('aqave',-1000.0d0,1000.0d0,aqave)
        call rdsdor ('aqamp',0.0d0,1000.0d0, aqamp)
        call rdsinr ('aqtamx',1,366,nr)
        aqtamx = fromto (bayrd,bayry,nr,year)+1
        call rdsdor ('aqper',0.0d0,366.0d0,aqper)
        aqomeg = pi*2.0d0/365.0d0
        if (aqamp.gt.0.1d0) then
          aqomeg = pi*2.d0/aqper
        endif


C --- flux-groundwater level relationship 
      else if (swbotb.eq.4) then
        call rdsdor ('cofqha',-100.0d0,100.0d0,cofqha)
        call rdsdor ('cofqhb',-1.0d0,1.0d0,cofqhb)


c --- pressure head of lowest compartment is given 
      else if (swbotb.eq.5) then
C -     data from files for each subrun/year
        if (sw1fil.eq.0) then
          call rdainr ('dd_bbc5',1,31,dd,366,ifnd)
          call rdfinr ('mm_bbc5',1,12,mm,366,ifnd)
          call rdfdor ('presshead',-1.0d10,1.0d10,presshead,mabbc,ifnd)
          nday = ifnd
C -     data from one file
        else if (sw1fil.eq.1) then
c         first year only
          if (runnr.eq.1 .or. iseq.eq.0) then
            call rdatim ('date_bbc5',date_bbc5,mabbc,ifnd)
            call rdfdor('presshead',-1.0d10,1.0d10,presshead,mabbc,ifnd)
          end if
C -       read data from arrays
          nday = 0
          do i = 1,ifnd
            call dtdpar(date_bbc5(i),datea,fsec)
            if ((datea(1).eq.bayry .and. datea(2).ge.fmay) .or. 
     &          (datea(1).eq.(bayry+1) .and. datea(2).lt.fmay)) then
              nday = nday + 1
              yy(nday) = datea(1)
              mm(nday) = datea(2)
              dd(nday) = datea(3)
              value(nday) = presshead(i)
            end if
          end do
        end if
c -     store values in hgitab
        do i = 1,nday
          if (mm(i).ge.fmay) yy(i) = bayry
          if (mm(i).lt.fmay) yy(i) = bayry+1
        end do
        do i = 1,nday
          call datonr (yy(i),mm(i),dd(i),nr)
          hgitab(i*2)   = value(i) 
          hgitab(i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
        end do

c --- zero flux
      else if (swbotb.eq.6) then
        continue

C --- free drainage 
      else if (swbotb.eq.7) then
        continue

C --- lysimeter with free outflow 
      else if (swbotb.eq.8) then
        continue

C --- combination of pressure head and flux
      else if (swbotb.eq.9) then

        call rdsdor ('shape_9',0.0d0,1.0d0,shape)
        call rdsdor ('hdrain_9',-1.0d4,0.0d0,hdrain)

c --    aqave: read table
C -     data from files for each subrun/year
        if (sw1fil.eq.0) then
          call rdainr ('dd_bbc9a',1,31,dd,366,ifnd)
          call rdfinr ('mm_bbc9a',1,12,mm,366,ifnd)
          call rdfdor ('HeadAqAve',-1.0d3,1.0d3,HeadAqAve,mabbc,ifnd)
          nday = ifnd
C -     data from one file
        else if (sw1fil.eq.1) then
c         first year only
          if (runnr.eq.1 .or. iseq.eq.0) then
            call rdatim ('date_bb9a',date_bbc9a,mabbc,ifnd)
            call rdfdor ('HeadAqAve',-1.0d3,1.0d3,HeadAqAve,mabbc,ifnd)
          end if
C -       read data from arrays
          nday = 0
          do i = 1,ifnd
            call dtdpar(date_bbc9a(i),datea,fsec)
            if ((datea(1).eq.bayry .and. datea(2).ge.fmay) .or. 
     &          (datea(1).eq.(bayry+1) .and. datea(2).lt.fmay)) then
              nday = nday + 1
              yy(nday) = datea(1)
              mm(nday) = datea(2)
              dd(nday) = datea(3)
              value(nday) = HeadAqAve(i)
            end if
          end do
        end if
c -     store values
        do i = 1,nday
          if (mm(i).ge.fmay) yy(i) = bayry
          if (mm(i).lt.fmay) yy(i) = bayry+1
        end do
        do i = 1,nday
          call datonr (yy(i),mm(i),dd(i),nr)
          aqatab(i*2)   = value(i) 
          aqatab(i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
        end do
c --    Reciprocal of resistance
        call rdsdor ('recires',0.0d0,1.0d5,recires)

c --    qbot: read table, position pointer
C -     data from files for each subrun/year
        if (sw1fil.eq.0) then
          call rdainr ('dd_bbc9b',1,31,dd,366,ifnd)
          call rdfinr ('mm_bbc9b',1,12,mm,366,ifnd)
          call rdfdor ('Qbot',-10.0d0,10.0d0,Qbot,mabbc,ifnd)
          nday = ifnd
C -     data from one file
        else if (sw1fil.eq.1) then
c         first year only
          if (runnr.eq.1 .or. iseq.eq.0) then
            call rdatim ('date_bb9b',date_bbc9b,mabbc,ifnd)
            call rdfdor ('Qbot',-1.0d3,1.0d3,Qbot,mabbc,ifnd)
          end if
C -       read data from arrays
          nday = 0
          do i = 1,ifnd
            call dtdpar(date_bbc9b(i),datea,fsec)
            if ((datea(1).eq.bayry .and. datea(2).ge.fmay) .or. 
     &          (datea(1).eq.(bayry+1) .and. datea(2).lt.fmay)) then
              nday = nday + 1
              yy(nday) = datea(1)
              mm(nday) = datea(2)
              dd(nday) = datea(3)
              value(nday) = Qbot(i)
            end if
          end do
        end if
c -     store values
        do i = 1,nday
          if (mm(i).ge.fmay) yy(i) = bayry
          if (mm(i).lt.fmay) yy(i) = bayry+1
        end do
        do i = 1,nday
          call datonr (yy(i),mm(i),dd(i),nr)
          qbotab(i*2)   = value(i) 
          qbotab(i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
        end do
      endif

      close (bbc)
      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE RDCRPD (CRPFIL,ENVSTR,SWCF,CFTB,IDSL,DLO,DLC,TSUMEA,
     &  TSUMAM,DTSMTB,DVSEND,TDWI,LAIEM,RGRLAI,SLATB,SPA,
     &  SSA,SPAN,TBASE,KDIF,KDIR,EFF,AMAXTB,TMPFTB,TMNFTB,CVL,CVO,CVR,
     &  CVS,Q10,RML,RMO,RMR,RMS,RFSETB,FRTB,FLTB,FSTB,FOTB,PERDL,RDRRTB,
     &  RDRSTB,HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,
     &  ADCRL,ECMAX,ECSLOP,COFAB,RDI,RRI,RDC,RDCTB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 6/6/01    
C     Purpose            : get crop parameters from cropfile
C     Subroutines called : 
C     Functions called   : -
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER CRP,I,IDSL,LOGF,IFND,SWCF
      integer*4 getun2
 
      REAL*8 CFTB(30),DTSMTB(30),SLATB(30),AMAXTB(30),TMPFTB(30)
      REAL*8 TMNFTB(30),RFSETB(30),FRTB(30),FLTB(30),FSTB(30),FOTB(30)
      REAL*8 ECMAX,ECSLOP,RDRRTB(30),RDRSTB(30),KDIF,KDIR,COFAB,LAIEM
      REAL*8 ADCRH,ADCRL,CVL,CVO,CVR,CVS,DLC,DLO,DVSEND,EFF
      REAL*8 PERDL,Q10,RDC,RDI,RGRLAI,RML,RMO,RMR,RMS,RRI,SPA,SPAN
      REAL*8 SSA,TBASE,TDWI,TSUMAM,TSUMEA,RDCTB(22)
      REAL*8 HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC
      REAL*8 X,SUM,RFAC,VALUE,AFGEN

      CHARACTER CRPFIL*16,ENVSTR*50,FILNAM*80


C ----------------------------------------------------------------------

C --- initialise and start reading
      filnam = trim(envstr)//trim(crpfil)//'.crp'
      crp = getun2 (10,19,2)
      call rdinit(crp,logf,filnam)

C --- Crop factor or crop height
      call rdsinr ('swcf',1,2,swcf)
      if (swcf.eq.1) then
        call rdador ('cftb',0.0d0,2.0d0,cftb,30,ifnd)
      elseif (swcf.eq.2) then
        call rdador ('cftb',0.0d0,5000.0d0,cftb,30,ifnd)
      endif

C --- phenology
      call rdsinr ('idsl',0,2,idsl)
      if (idsl.eq.1.or.idsl.eq.2) then
        call rdsdor ('dlo',0.0d0,24.0d0,dlo)
        call rdsdor ('dlc',0.0d0,24.0d0,dlc)
      endif
      if (idsl.eq.0.or.idsl.eq.2) then
        call rdsdor ('tsumea',0.0d0,10000.0d0,tsumea)
        call rdsdor ('tsumam',0.0d0,10000.0d0,tsumam)
      endif
      call rdador ('dtsmtb',0.0d0,100.0d0,dtsmtb,30,ifnd) 
      call rdsdor ('dvsend',0.0d0,3.0d0,dvsend)

C --- initial
      call rdsdor ('tdwi',0.0d0,10000.0d0,tdwi)
      call rdsdor ('laiem',0.0d0,10.0d0,laiem)
      call rdsdor ('rgrlai',0.0d0,1.0d0,rgrlai)

C --- green area
      call rdador ('slatb',0.0d0,2.0d0,slatb,30,ifnd)
      call rdsdor ('spa',0.0d0,1.0d0,spa)
      call rdsdor ('ssa',0.0d0,1.0d0,ssa)
      call rdsdor ('span',0.0d0,366.0d0,span)
      call rdsdor ('tbase',-10.0d0,30.0d0,tbase)

C --- assimilation
      call rdsdor ('kdif',0.0d0,2.0d0,kdif)
      call rdsdor ('kdir',0.0d0,2.0d0,kdir)
      call rdsdor ('eff',0.0d0,10.0d0,eff)
      call rdador ('amaxtb',0.0d0,100.0d0,amaxtb,30,ifnd)
      call rdador ('tmpftb',-10.0d0,50.0d0,tmpftb,30,ifnd)
      call rdador ('tmnftb',-10.0d0,50.0d0,tmnftb,30,ifnd)

C --- conversion of assimilates into biomass
      call rdsdor ('cvl',0.0d0,1.0d0,cvl)
      call rdsdor ('cvo',0.0d0,1.0d0,cvo)
      call rdsdor ('cvr',0.0d0,1.0d0,cvr)
      call rdsdor ('cvs',0.0d0,1.0d0,cvs)

C --- maintenance respiration
      call rdsdor ('q10',0.0d0,5.0d0,q10)
      call rdsdor ('rml',0.0d0,1.0d0,rml)
      call rdsdor ('rmo',0.0d0,1.0d0,rmo)
      call rdsdor ('rmr',0.0d0,1.0d0,rmr)
      call rdsdor ('rms',0.0d0,1.0d0,rms)
      call rdador ('rfsetb',0.0d0,3.0d0,rfsetb,30,ifnd)

C --- partitioning
      call rdador ('frtb',0.0d0,3.0d0,frtb,30,ifnd)
      call rdador ('fltb',0.0d0,3.0d0,fltb,30,ifnd)
      call rdador ('fstb',0.0d0,3.0d0,fstb,30,ifnd)
      call rdador ('fotb',0.0d0,3.0d0,fotb,30,ifnd)

C --- death rates
      call rdsdor ('perdl',0.0d0,3.0d0,perdl)
      call rdador ('rdrrtb',0.0d0,3.0d0,rdrrtb,30,ifnd)
      call rdador ('rdrstb',0.0d0,3.0d0,rdrstb,30,ifnd)

C --- water use
      call rdsdor ('hlim1' ,-100.0d0,100.0d0,hlim1)
      call rdsdor ('hlim2u',-1000.0d0,100.0d0,hlim2u)
      call rdsdor ('hlim2l',-1000.0d0,100.0d0,hlim2l)
      call rdsdor ('hlim3h',-10000.0d0,100.0d0,hlim3h)
      call rdsdor ('hlim3l',-10000.0d0,100.0d0,hlim3l)
      call rdsdor ('hlim4' ,-16000.0d0,100.0d0,hlim4)
      call rdsdor ('rsc',0.0d0,1000.0d0,rsc)
      call rdsdor ('adcrh',0.0d0,5.0d0,adcrh)
      call rdsdor ('adcrl',0.0d0,5.0d0,adcrl)
      call rdsdor ('ecmax',0.0d0,20.0d0,ecmax)
      call rdsdor ('ecslop',0.0d0,40.0d0,ecslop)
      call rdsdor ('cofab',0.0d0,1.0d0,cofab)

C --- rooting
      call rdador ('rdctb',0.0d0,100.0d0,rdctb,22,ifnd)
      call rdsdor ('rdi',0.0d0,1000.0d0,rdi)
      call rdsdor ('rri',0.0d0,100.0d0,rri)
      call rdsdor ('rdc',0.0d0,1000.0d0,rdc)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 I = 5,100,10
        X = 0.01d0*dble(I)
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
   96 CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
        RDCTB(I*2) = RDCTB(I*2)*RFAC
   98 continue

      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDCRPS (CRPFIL,ENVSTR,IDEV,LCC,TSUMEA,TSUMAM,TBASE,
     & KDIF,KDIR,GCTB,SWGC,CFTB,SWCF,RDTB,KYTB,HLIM1,HLIM2U,HLIM2L,
     & HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,ADCRL,ECMAX,ECSLOP,RDCTB,
     & COFAB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 6/6/01             
C     Purpose            : get crop parameters from cropfile
C     Subroutines called : 
C     Functions called   : AFGEN
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   CRP,IDEV,I,LCC,SWGC,SWCF,LOGF,IFND
      integer*4 getun2

      REAL*8    GCTB(72),CFTB(72),RDTB(72),KYTB(72),RDCTB(22)
      REAL*8    ADCRH,ADCRL,ECMAX,ECSLOP,TBASE,TSUMAM,TSUMEA
      REAL*8    HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC
      REAL*8    X,SUM,VALUE,RFAC,AFGEN,KDIF,KDIR,COFAB

      CHARACTER CRPFIL*16,FILNAM*80,ENVSTR*50

      LOGICAL   EXISTS
C ----------------------------------------------------------------------

C --- initialise and start reading
      filnam = trim(envstr)//trim(crpfil)//'.crp'
      crp = getun2 (10,19,2)
      call rdinit(crp,logf,filnam)

C --- phenology
      call rdsinr ('idev',1,2,idev)
      if (idev.eq.1) then
        call rdsinr ('lcc',1,366,lcc)
      elseif (idev.eq.2) then
        call rdsdor ('tsumea',0.0d0,10000.0d0,tsumea)
        call rdsdor ('tsumam',0.0d0,10000.0d0,tsumam)
        call rdsdor ('tbase',-10.0d0, 20.0d0,tbase)
      endif

C --- assimilation                        
      call rdsdor ('kdif',0.0d0,2.0d0,kdif)
      call rdsdor ('kdir',0.0d0,2.0d0,kdir)
     
C --- LAI or soil cover fraction 
      call rdsinr ('swgc',1,2,swgc)
      if (swgc.eq.1) then
        call rdador ('gctb',0.0d0,12.0d0,gctb,72,ifnd)
      elseif (swgc.eq.2) then
        call rdador ('gctb',0.0d0,2.0d0,gctb,72,ifnd)
      endif

C --- Crop factor or crop height
      call rdsinr ('swcf',1,2,swcf)
      if (swcf.eq.1) then
        call rdador ('cftb',0.0d0,2.0d0,cftb,72,ifnd)
      elseif (swcf.eq.2) then
        call rdador ('cftb',0.0d0,5000.0d0,cftb,72,ifnd)
      endif

C --- rooting depth
      call rdador ('rdtb',0.0d0,1000.0d0,rdtb,72,ifnd)

C --- yield response
      call rdador ('kytb',0.0d0,5.0d0,kytb,72,ifnd)

C --- water use
      call rdsdor ('hlim1' ,-100.0d0,100.0d0,hlim1)
      call rdsdor ('hlim2u',-1000.0d0,100.0d0,hlim2u)
      call rdsdor ('hlim2l',-1000.0d0,100.0d0,hlim2l)
      call rdsdor ('hlim3h',-10000.0d0,100.0d0,hlim3h)
      call rdsdor ('hlim3l',-10000.0d0,100.0d0,hlim3l)
      call rdsdor ('hlim4' ,-16000.0d0,100.0d0,hlim4)
      call rdsdor ('rsc',0.0d0,1000.0d0,rsc)
      call rdsdor ('adcrh',0.0d0,5.0d0,adcrh)
      call rdsdor ('adcrl',0.0d0,5.0d0,adcrl)
      call rdsdor ('ecmax',0.0d0,20.0d0,ecmax)
      call rdsdor ('ecslop',0.0d0,40.0d0,ecslop)
      call rdsdor ('cofab',0.0d0,1.0d0,cofab)

C --- read table with root distribution coefficients
      call rdador ('rdctb',0.0d0,100.0d0,rdctb,22,ifnd)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 I = 5,100,10
        X = 0.01d0*dble(I)
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
   96 CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
        RDCTB(I*2) = RDCTB(I*2)*RFAC
   98 continue

C --- close file with crop data
      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDDRB (DRBFIL,ENVSTR,BAYRD,BAYRY,DRAMET,
     &IPOS,BASEGW,KHTOP,KHBOT,KVTOP,KVBOT,ZINTF,L,WETPER,ZBOTDR,ENTRES,
     &GEOFAC,NRLEVS,DRARES,INFRES,SWALLO,SWDTYP,OWLTAB,QDRTAB,LOGF,FMAY,
     &SWDIVD)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                        
C     Purpose            : reading drainage parameters    
C                          for one agricultural year;  
C     Subroutines called : DATONR, 
C     Functions called   : FROMTO                                      
C     File usage         : DRBFIL, file containing drainage data
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER   SWALLO(5),SWDTYP(5),FROMTO,BAYRD,BAYRY,FMAY
      INTEGER   DRAMET,DRB,NR,IPOS,NRLEVS,LOGF,SWDIVD

      REAL*8    L(5),ZBOTDR(5),OWLTAB(5,2*MAOWL),DRARES(5),INFRES(5)
      REAL*8    QDRTAB(50)
      REAL*8    BASEGW(2),WETPER(5),KHTOP,KHBOT,KVTOP,KVBOT
      REAL*8    ZINTF,ENTRES,GEOFAC

      CHARACTER DRBFIL*16,ENVSTR*50,FILNAM*80
C ----------------------------------------------------------------------
C --- local
      INTEGER  dd(MAOWL),mm(MAOWL),yy(MAOWL),IFND,I
      integer*4 getun2
      REAL*8   level(MAOWL),q(25),gwl(25)

C ----------------------------------------------------------------------

C --- open file with drainage data
      filnam = trim(envstr)//trim(drbfil)//'.drb'
      drb = getun2 (10,19,2)
      call rdinit(drb,logf,filnam)

C --- method to establish drainage/infiltration fluxes
      call rdsinr ('dramet',1,3,dramet) 

      if (dramet.ne.3) nrlevs = 1

      if (dramet.eq.1) then  
        call rdsdor ('lm1',1.0d0,1000.d0,l(1))
        call rdador ('gwl',-10000.0d0,10.0d0,gwl,25,ifnd)
        call rdfdor ('q',0.0d0,1000.0d0,q,25,ifnd)

        l(1) = 100.0d0*l(1)
        do 7 i = 1,50
          qdrtab(i) = 0.0
    7   continue
        do 9 i = 1,ifnd
          qdrtab(i*2-1) = abs(gwl(i))
          qdrtab(i*2)   = q(i)
    9   continue

      elseif (dramet.eq.2) then
   
C ---   read profile characteristics
        call rdsinr ('ipos',1,5,ipos)

        call rdsdor ('basegw',-1.0d4,0.0d0,basegw(1))
        call rdsdor ('khtop',0.0d0,1000.0d0,khtop)

        if (ipos.ge.3) then
          call rdsdor ('khbot',0.0d0,1000.0d0,khbot)
          call rdsdor ('zintf',-1.0d4,0.0d0,zintf)
        endif
        if (ipos.ge.4) then
          call rdsdor ('kvtop',0.0d0,1000.0d0,kvtop)
          call rdsdor ('kvbot',0.0d0,1000.0d0,kvbot)  
        endif
        if (ipos.eq.5) then
          call rdsdor ('geofac',0.0d0,100.0d0,geofac)
        endif

C ---   read drain characteristics
        call rdsdor ('lm2',1.0d0,1000.0d0,l(1))
        l(1) = 100.0d0*l(1)
        call rdsdor ('wetper',0.0d0,1000.0d0,wetper(1))
        call rdsdor ('zbotdr',-1000.0d0,0.0d0,zbotdr(1))
        call rdsdor ('entres',0.0d0,1000.0d0,entres)

      elseif (dramet.eq.3) then
   
        call rdsinr ('nrlevs',1,5,nrlevs)

        if (nrlevs.ge.1) then
          call rdsdor ('drares1',10.0d0,1.0d5,drares(1))
          call rdsdor ('infres1',0.0d0,1.0d5,infres(1))
          call rdsinr ('swallo1',1,3,swallo(1))
          if (swdivd .eq. 1) then
            call rdsdor ('l1',1.0d0,1000.0d0,l(1))
          endif
          l(1) = 100.0d0*l(1)
          call rdsdor ('zbotdr1',-1000.0d0,0.0d0,zbotdr(1))
          call rdsinr ('swdtyp1',1,2,swdtyp(1))
          if (swdtyp(1).eq.2) then
            call rdainr ('dd_drb1',1,31,dd,maowl,ifnd)
            call rdfinr ('mm_drb1',1,12,mm,maowl,ifnd)
            call rdfdor ('level1',-1000.0d0,10.0d0,level,maowl,ifnd)
C -         store values
            do 16 i = 1,ifnd 
              if (mm(i).ge.fmay) yy(i) = bayry
              if (mm(i).lt.fmay) yy(i) = bayry+1
              call datonr (yy(i),mm(i),dd(i),nr)
              owltab(1,i*2)   = -abs(level(i))
              owltab(1,i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
16          continue
          endif
        endif 

        if (nrlevs.ge.2) then
          call rdsdor ('drares2',10.0d0,1.0d5,drares(2))
          call rdsdor ('infres2',0.0d0,1.0d5,infres(2))
          call rdsinr ('swallo2',1,3,swallo(2))
          if (swdivd .eq. 1) then
            call rdsdor ('l2',1.0d0,1000.0d0,l(2))
          endif
          L(2) = 100.0d0*L(2)
          call rdsdor ('zbotdr2',-1000.0d0,0.0d0,zbotdr(2))
          call rdsinr ('swdtyp2',1,2,swdtyp(2))
          if (swdtyp(2).eq.2) then
            call rdainr ('dd_drb2',1,31,dd,maowl,ifnd)
            call rdfinr ('mm_drb2',1,12,mm,maowl,ifnd)
            call rdfdor ('level2',-1000.0d0,10.0d0,level,maowl,ifnd)
C -         store values
            do 26 i = 1,ifnd 
              if (mm(i).ge.fmay) yy(i) = bayry
              if (mm(i).lt.fmay) yy(i) = bayry+1
              call datonr (yy(i),mm(i),dd(i),nr)
              owltab(2,i*2)   = -abs(level(i))
              owltab(2,i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
26          continue
          endif
        endif 

        if (nrlevs.ge.3) then
          call rdsdor ('drares3',10.0d0,1.0d5,drares(3))
          call rdsdor ('infres3',0.0d0,1.0d5,infres(3))
          call rdsinr ('swallo3',1,3,swallo(3))
          if (swdivd .eq. 1) then
            call rdsdor ('l3',1.0d0,1000.0d0,l(3))
          endif
          l(3) = 100.0d0*l(3)
          call rdsdor ('zbotdr3',-1000.0d0,0.0d0,zbotdr(3))
          call rdsinr ('swdtyp3',1,2,swdtyp(3))
          if (swdtyp(3).eq.2) then
            call rdainr ('dd_drb3',1,31,dd,maowl,ifnd)
            call rdfinr ('mm_drb3',1,12,mm,maowl,ifnd)
            call rdfdor ('level3',-1000.0d0,10.0d0,level,maowl,ifnd)
C -         store values
            do 36 i = 1,ifnd 
              if (mm(i).ge.fmay) yy(i) = bayry
              if (mm(i).lt.fmay) yy(i) = bayry+1
              call datonr (yy(i),mm(i),dd(i),nr)
              owltab(3,i*2)   = -abs(level(i))
              owltab(3,i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
36          continue
          endif
        endif 

        if (nrlevs.ge.4) then
          call rdsdor ('drares4',10.0d0,1.0d5,drares(4))
          call rdsdor ('infres4',0.0d0,1.0d5,infres(4))
          call rdsinr ('swallo4',1,3,swallo(4))
          if (swdivd .eq. 1) then
            call rdsdor ('l4',1.0d0,1000.0d0,l(4))
          endif
          l(4) = 100.0d0*l(4)
          call rdsdor ('zbotdr4',-1000.0d0,0.0d0,zbotdr(4))
          call rdsinr ('swdtyp4',1,2,swdtyp(4))
          if (swdtyp(4).eq.2) then
            call rdainr ('dd_drb4',1,31,dd,maowl,ifnd)
            call rdfinr ('mm_drb4',1,12,mm,maowl,ifnd)
            call rdfdor ('level4',-1000.0d0,10.0d0,level,maowl,ifnd)
C -         store values
            do 46 i = 1,ifnd 
              if (mm(i).ge.fmay) yy(i) = bayry
              if (mm(i).lt.fmay) yy(i) = bayry+1
              call datonr (yy(i),mm(i),dd(i),nr)
              owltab(4,i*2)   = -abs(level(i))
              owltab(4,i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
46          continue
          endif
        endif 

        if (nrlevs.ge.5) then
          call rdsdor ('drares5',10.0d0,1.0d5,drares(5))
          call rdsdor ('infres5',0.0d0,1.0d5,infres(5))
          call rdsinr ('swallo5',1,3,swallo(2))
          if (swdivd .eq. 1) then
            call rdsdor ('l5',1.0d0,1000.0d0,l(5))
          endif
          l(5) = 100.0d0*l(5)
          call rdsdor ('zbotdr5',-1000.0d0,0.0d0,zbotdr(5))
          call rdsinr ('swdtyp5',1,2,swdtyp(5))
          if (swdtyp(5).eq.2) then
            call rdainr ('dd_drb5',1,31,dd,maowl,ifnd)
            call rdfinr ('mm_drb5',1,12,mm,maowl,ifnd)
            call rdfdor ('level5',-1000.0d0,10.0d0,level,maowl,ifnd)
C -         store values
            do 56 i = 1,ifnd 
              if (mm(i).ge.fmay) yy(i) = bayry
              if (mm(i).lt.fmay) yy(i) = bayry+1
              call datonr (yy(i),mm(i),dd(i),nr)
              owltab(5,i*2)   = -abs(level(i))
              owltab(5,i*2-1) = fromto (bayrd,bayry,nr,yy(i))+1
56          continue
          endif
        endif 
      endif 

C --- close file with drainage data
      close (drb)         

      RETURN
      END


C ----------------------------------------------------------------------
      SUBROUTINE RDDRE (SRFFIL,ENVSTR,RSRO,NRSRF,NRPRI,NRSEC,L,
     & ZBOTDR,WIDTHR,TALUDR,RDRAIN,RINFI,RENTRY,REXIT,GWLINF,SWDTYP,
     & WLPTAB,BAYRD,BAYRY,SWSEC,WLS1,OSSWLM,NMPER,WLSTAR,IMPEND,SWMAN,
     & WSCAP,SWQHR,HBWEIR,ALPHAW,BETAW,NQH,HQHTAB,QQHTAB,DROPR,WLSMAN,
     & GWLCRIT,WLSTAB,STTAB,WSTINI,WST,T,WLSBAK,SWSRF,NPHASE,HCRIT,
     & HDEPTH,VCRIT,WLP1,NODHD,NUMNOD,DZ,WLDIP,NUMADJ,LOGF,FMAY,INTWL,
     & SWNRSRF,RSURFDEEP,RSURFSHALLOW)
C ----------------------------------------------------------------------
C     Date               : 5/6/2001                   
C     Purpose            : reading multilevel drainage characteristics 
C                          and specification of the surface water system
C                          for a period up to one year;  

C --- 1 Reading .DRE input file
C --- 2 Initializations

C --- Initializations:
C -1- wlp1: water level in primary system   (SWSRF = 3)
C -2- wls1: water level in secondary system (SWSRF = 2 or 3, SWSEC = 1 or 2) 
C -3- HBWEIR(IMPER) in case of table discharge relation (SWQHR = 2)
C -4- NUMADJ: number of target level adjustments
C -5- sttab(22,2) table with storage as a function of water level
C ---    sttab(i,1) contains levels:
C ---    i=1: +100 cm; i=2: 0 cm; i=22: bottom of deepest dr. medium
C ---    sttab(i,2) contains storage expressed as surface layer 

C     Subroutines called :  
C     Functions called   :                               
C     File usage         : DREFIL, file containing drainage data
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER   NRPRI,NRSEC,NRSRF,SWDTYP(5),NMPER,SWMAN(10),IMPEND(10)
      INTEGER   SWQHR,NQH(10),BAYRD,BAYRY,SWSEC,SWSRF,NPHASE(10)
      INTEGER   NODHD(10),NUMNOD,NUMADJ,LOGF,FMAY,INTWL(10),SWNRSRF

      REAL*8    L(5),ZBOTDR(5),WIDTHR(5),TALUDR(5),RDRAIN(5)
      REAL*8    RINFI(5),RENTRY(5),REXIT(5),GWLINF(5),WLPTAB(2*MAWLP)
      REAL*8    RSRO,RSURFDEEP,RSURFSHALLOW
      REAL*8    WLS1,OSSWLM,WLSTAR,WSCAP(10),HBWEIR(10),ALPHAW(10)
      REAL*8    BETAW(10),HQHTAB(10,10),QQHTAB(10,10),DROPR(10)
      REAL*8    WLSMAN(10,10),WLSTAB(2*MAWLS),STTAB(22,2),T,HDEPTH(10)
      REAL*8    WSTINI,WST,WLSBAK(4),HCRIT(10,10),VCRIT(10,10)
      REAL*8    AFGEN,WLP1,DZ(MACP),GWLCRIT(10,10),WLDIP(10)

      CHARACTER SRFFIL*16,ENVSTR*50

C ----------------------------------------------------------------------
C --- local
      INTEGER   DRE,level(5),D,M,Y,NR,FROMTO,ITAB,I, ilev
      INTEGER   iph,NRMAN1,NRMAN2,NODE,IMPER,IMPERB,IMPERI,IFND
      integer*4 getun2,dd_wl(mawlp),mm_wl(mawlp)
      integer*4 dd_mp(10),mm_mp(10),imper_4b(10),imper_4c(10)
      integer*4 imper_4e1(10),imper_4e2(10*10),impphase(10*10)
      integer*4 imper_4d(10*10),imptab(10*10)
   
      REAL*8    wlp(mawlp),wls(mawlp),zb, htab(10*10),qtab(10*10)
      REAL*8    wlsman_t(10*10),gwlcrit_t(10*10),hcrit_t(10*10)
      REAL*8    vcrit_t(10*10)
      REAL*8    wdepth,wbreadth,wvolum,DEP,WSTLEV,SOFCU,ALTCU

      LOGICAL   FLWEIR(10),EXISTS(10),FLZERO(10),BLANK,COMMENT

      CHARACTER FILNAM*80

	INTEGER GetErrorStatus
	EXTERNAL GetErrorStatus

C ----------------------------------------------------------------------
C --- open file with drainage data
      filnam = trim(envstr)//trim(srffil)//'.dre'
      dre = getun2 (10,19,2)
      call rdinit(dre,logf,filnam)

C --- altitude of control unit (relative to reference level)
      call rdsdor ('altcu',-3.0d5,3.0d5,altcu)

C --- section 1

C --  drainage resistance of surface runoff
      call rdsdor ('rsro',0.001d0,1.0d0,rsro)

C --  number of drainage levels
      call rdsinr ('nrsrf',1,5,nrsrf)

C --- characteristics of each drainage level 
      call rdfinr ('lev',1,5,level,5,nrsrf)
      call rdfinr ('swdtyp',0,1,swdtyp,5,nrsrf)
      call rdfdor ('l',1.0d0,100000.0d0,l,5,nrsrf)
      call rdfdor ('zbotdr',(altcu-1.0d3),(altcu-1.0d-2),zbotdr,5,nrsrf)
      call rdfdor ('gwlinf',-10000.0d0,0.0d0,gwlinf,5,nrsrf)
      call rdfdor ('rdrain',1.0d0,1.0d5,rdrain,5,nrsrf)
      call rdfdor ('rinfi',1.0d0,1.0d5,rinfi,5,nrsrf)
      call rdfdor ('rentry',0.0d0,10.0d0,rentry,5,nrsrf)
      call rdfdor ('rexit',0.0d0,10.0d0,rexit,5,nrsrf)
      call rdfdor ('widthr',0.0d0,10000.0d0,widthr,5,nrsrf)
      call rdfdor ('taludr',1.0d-2,5.0d0,taludr,5,nrsrf)

C -   conversions and security checks....
      do i = 1, nrsrf
C       conversions
        l(i)      = l(i)*100.0d0
        zbotdr(i) = zbotdr(i) -altcu
C -     security checks....
C -     0 - verify levels-index
C -     1 - levels must be ordered, starting with the deepest
C -     2 - zbotdr always below surface level
C -     3 - gwlinf must be below bottom of deepest drainage medium
        if (level(i).ne.i) then
          write(*,*) 'RDDRE: dr. level index not consistent' 
          stop 
        endif
        if (i .gt. 1) then
          if (zbotdr(i) .lt. zbotdr(i-1)) then
            write(*,*) 
     &      'RDDRE: levels must be ordered, starting with deepest'
            stop 
          endif
        endif
        if (zbotdr(i).gt.0.0) then
          write(*,*) 
     &      'RDDRE: zbotdr must be negative'
          stop 
        endif
        if (gwlinf(i).gt.zbotdr(i)) then
          write(*,*) 
     &      'RDDRE: gwlinf higher than zbotdr'
          stop 
        endif
      enddo

C --- type of highest drainage level
	call rdsinr ('swnrsrf',0,1,swnrsrf)
      if (swnrsrf.eq.1) then
        call rdsdor ('rsurfdeep',0.001d0,1000.0d0,rsurfdeep)
        call rdsdor ('rsurfshallow',0.001d0,1000.0d0,rsurfshallow)
      endif

C --- section 2a
      call rdsinr ('swsrf',1,3,swsrf)
      if (swsrf.eq.3) then
        nrpri = 1
        nrsec = nrsrf-nrpri
      elseif (swsrf.eq.2) then
        nrpri = 0
        nrsec = nrsrf-nrpri
      elseif (swsrf.eq.1) then
c ---   no surface water system...ready
        close (dre)
        return
      endif

C --- is dit wel goed ?
      if (nrsec.lt.1) then
        write(*,*) 'rddre - secondary system not specified'
        stop
      endif
      if (swdtyp(1+nrpri).ne.0) then
        write(*,*) 'rddre - deepest sec. level must be open'
        stop
      endif

C --- section 2b

C --- read table with water levels in the primary system (SWSRF=3)
      IF (SWSRF.EQ.3) THEN
C ---   init table
        DO 8 I = 1,2*MAWLP
          WLPTAB(I) = 0.0
    8   continue
C -     read and store
        call rdainr ('dd_dre1',1,31,dd_wl,mawlp,ifnd)
        call rdfinr ('mm_dre1',1,12,mm_wl,mawlp,ifnd)
        call rdfdor
     &        ('wlp',(altcu-1000.0d0),(altcu-0.01d0),wlp,mawlp,ifnd)
        do i = 1, ifnd
          wlptab(i*2) = wlp(i) -altcu
          if (wlptab(i).gt.0.0) then
            write(*,*) 
     &       'rddre: water level in primary system must be negative'
            stop
          endif
          d = dd_wl(i)
          m = mm_wl(i)
          IF (M.GE.FMAY) Y = BAYRY
          IF (M.LT.FMAY) Y = BAYRY+1
          CALL DATONR (Y,M,D,NR)
          WLPTAB(I*2-1) = FROMTO (BAYRD,BAYRY,NR,Y)+1
        enddo
C ---   set initial value
        wlp1 = AFGEN (WLPTAB,2*MAWLP,T+1.0d0)

C ---   Ready in case of only a primary system ??
C       if (NRSEC.LT.1) then
C          CLOSE(DRE)
C          Return
C       endif 

      ENDIF

C --- Section 2c

      call rdsinr ('swsec',1,2,swsec)

C --- Section 3

      if (swsec.eq.1) then
C ---   surface water level of secondary system is input
C ---   position file pointer

C ---   init table
        do 60 i = 1,2*mawls
          wlstab(i) = 0.0
   60   continue
C -     read and store
        call rdainr ('dd_dre2',1,31,dd_wl,mawlp,ifnd)
        call rdfinr ('mm_dre2',1,12,mm_wl,mawlp,ifnd)
        call rdfdor
     &         ('wls',(altcu-1000.0d0),(altcu-0.01d0),wls,mawlp,ifnd)
        do i = 1, ifnd
          wlstab(i*2) = wls(i) -altcu
          d = dd_wl(i)
          m = mm_wl(i)
          if (m.ge.fmay) y = bayry
          if (m.lt.fmay) y = bayry+1
          call datonr (y,m,d,nr)
          wlstab(i*2-1) = fromto (bayrd,bayry,nr,y)+1
        enddo
c ---   set initial value
        wls1 = afgen (wlstab,2*mawls,t+1.0d0)

C ---   Section 4a

C --- surface water level of secondary system is simulated
      elseif (swsec.eq.2) then
        call rdsdor ('wlact',dble(zbotdr(1+nrpri)+altcu)
     &       ,dble(altcu),wls1)
        wls1 = wls1 - altcu
        call rdsdor ('osswlm',0.0d0,10.0d0,osswlm)

c ---   section 4b

        call rdsinr ('nmper',1,10,nmper)
        wlstar = wls1

        call rdfinr ('imper_4b',1,nmper,imper_4b,10,nmper)
        call rdfinr ('dd_MP',1,31,dd_mp,10,nmper)
        call rdfinr ('mm_MP',1,12,mm_mp,10,nmper)
        call rdfinr ('swman',1,2,swman,10,nmper)
        call rdfdor ('wscap',0.0d0,10.0d0,wscap,10,nmper)
        call rdfdor ('wldip',0.0d0,100.0d0,wldip,10,nmper)
        call rdfinr ('intwl',1,31,intwl,10,nmper)

C -     for each type of management: count number of periods 
        nrman1 = 0
        nrman2 = 0
        do imper = 1, nmper
          if (swman(imper).eq.2 .and. intwl(imper).lt.1) then
            write(*,*)'rddre: intwl (management interval) must be >= 1d'
            stop
          endif
          wldip(imper) = abs(wldip(imper))
          if (swman(imper).eq.1) then 
            nrman1 = nrman1+1
          elseif (swman(imper).eq.2) then
            nrman2 = nrman2+1
          else
            write(*,*) 'rddre: swman out of range'
            stop
          endif
C ---     calculate year 
          d = dd_mp(imper)
          m = mm_mp(imper)
          if (m.ge.fmay) y = bayry
          if (m.lt.fmay) y = bayry+1
          call datonr (y,m,d,nr)
          impend(imper) = fromto (bayrd,bayry,nr,y)+1
        enddo
        if ((nrman1+nrman2).ne.nmper) then
          write(*,*) 'rddre: nrman1+nrman2 does not match nmper'
          stop
        endif

C ---   type of discharge relationship
        call rdsinr ('swqhr',1,2,swqhr)

        IF (SWQHR.EQ.1) THEN

C ---     Section 4c

          call rdsdor ('sofcu',0.1d0 ,100000.0d0,sofcu)

          call rdfinr ('imper_4c',1,nmper,imper_4c,10,nmper)
          zb = min(zbotdr(1),zbotdr(2))
          call rdfdor 
     &            ('hbweir',(altcu+zb),(altcu+100.0d0),hbweir,10,nmper)
          call rdfdor ('alphaw',0.1d0,50.0d0,alphaw,10,nmper)
          call rdfdor ('betaw',0.5d0,3.0d0,betaw,10,nmper)

c ---     convert and check values

c         initialise
          imperb = 0
          imperi = 0

          do imper = 1, nmper
            hbweir(imper)= hbweir(imper)-ALTCU
C ---       correction for units
            alphaw(imper) = alphaw(imper)*(8.64*100**
     &                      (1.0d0-betaw(imper))/SOFCU)     
            if (hbweir(imper).lt.zbotdr(1+NRPRI)) then
              write(6,*) 'RDDRE - weir crest level below bottom of'
              write(6,*) 'deepest channel of secondary system'
              stop
            endif
c --- check for target level above channel bottom when supply is 
c --- attempted (system may never become dry in this case)
            if (swman(imper).eq.1 .and. wscap(imper).gt.1.d-7 .and.
     &         (hbweir(imper)-wldip(imper)).lt.
     &         (zbotdr(1+NRPRI)+1.d-4)) then
              write(6,*) 'RDDRE: HBWEIR/WLDIP !'
              write(6,*) '       supply not possible =< zbotdr !'
              stop
            endif

            if (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            else
              write(6,*) 'rddre: 4c imper not unique'
              stop
            endif
          enddo

C ---     check number of records.. 
          if (imperi.ne.nmper) then
              write(6,*)'rddre: section 4c - not enough records'
              stop
          endif

        ELSEIF (SWQHR.EQ.2) THEN

C ---     Section 4d

c --      initialise
          IMPERB = 0
          IMPERI = 0
          DO 34 IMPER = 1,10
            NQH (IMPER)    = 0
            FLWEIR (IMPER) = .FALSE.
            EXISTS (IMPER) = .FALSE.
            FLZERO (IMPER) = .FALSE.
34        CONTINUE

c --      read and store
          call rdainr ('imper_4d',1,nmper,imper_4d,100,ifnd)
          call rdfinr ('imptab',1,nmper,imptab,100,ifnd)
          call rdfdor 
     &          ('htab',(altcu-1000.0d0),(altcu+100.0d0),htab,100,ifnd)
          call rdfdor ('qtab',0.0d0,500.0d0,qtab,100,ifnd)

c ---     convert
          do i = 1, ifnd
            hqhtab(imper_4d(i),imptab(i)) = htab(i) - Altcu
            qqhtab(imper_4d(i),imptab(i)) = qtab(i)
          enddo

c ---     check values
          do i = 1, ifnd
            imper = imper_4d(i)
            itab = imptab(i)
            NQH(IMPER) = NQH(IMPER)+1
            EXISTS(IMPER) = .TRUE.
            if (qqhtab(imper,itab).lt.0.000001d0) flzero(imper)=.true.
            if (hqhtab(imper,itab).lt.zbotdr(1+nrpri)) then
              write(6,*) 'rddre - level in q-h table below bottom of '
              write(6,*) 'deepest channel of secondary system'
              stop
            endif
            if (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            endif

c -         establish hbweir (at level where qtab = 0) 
            if (qqhtab(imper,itab).lt.0.0001d0.and..not.flweir(imper)) 
     &      then
              hbweir(imper) = hqhtab(imper,itab)
              flweir(imper) = .true.
            endif 

C -         consistency checks
            if (nqh(imper).ne.itab) then
              write(*,*) 'rddre: qh-table / imper - itab mismatch'
              stop
            endif
            if (itab.eq.1) then
              if (abs(hqhtab(imper,itab)-100.0d0).gt.0.00001d0) then
                write(*,*) 
     &            'rddre: first value in htab should be altcu+100.0'
                stop 'rddre: first value in htab should be altcu+100.0'
              endif
            endif
            if (itab.gt.1) then
              if ((hqhtab(imper,itab).ge.hqhtab(imper,itab-1)).or.
     &           (qqhtab(imper,itab).gt.qqhtab(imper,itab-1))) then
                write(*,*) 'rddre: qh-table - no descending values'
                stop 
              endif
            endif

          enddo

C ---     check number of periods.. 
          if (imperi.ne.nmper) then
            write(*,*) 'rddre: section 4d - number of periods incorrect'
            stop
          endif

C ---     check that QQHTAB goes down to zero
          do imper = 1,nmper
            if (exists(imper).and..not.flzero(imper)) then
              write(*,*) 'rddre: qqhtab not going down to zero'
              stop 
            endif
          enddo

        endif

C ---   Section 4e1

        IF (NRMAN2.GT.0) THEN

          IMPERB = 0
          IMPERI = 0

C ---     read table with drop rates 
          call rdfinr ('imper_4e1',1,nmper,imper_4e1,10,nmper)
          call rdfdor ('dropr',0.0d0,100.0d0,dropr,10,nmper)
          call rdfdor ('hdepth',-100d0,0.0d0,hdepth,10,nmper)

          do imper = 1, nmper
            hdepth(imper) = -abs(hdepth(imper))

C ---       determine compartment number related to hdepth
c .. ? verify kroes 18Mar2002
            dep = 0.0
            node = 1
            do while (hdepth(imper).ge.(dep-1.0d-6).and.node.le.numnod) 
              nodhd(imper) = node
              dep = dep-dz(node)
              node = node + 1
            enddo

            if (swman(imper) .ne. 2) then
              write(*,*) 'rddre: #4e swman - imper mismatch'
              stop 
            endif
            if (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            else
              write(*,*) 'rddre: two drop rates same period'
              stop 
            endif
          enddo
          if (imperi .ne. nrman2) then
            write(*,*)
     &       'rddre: #4e number of periods for drop rate incorrect'
            stop
          endif

C ---   Section 4e2

          IMPERB = 0
          IMPERI = 0

          DO 54 IMPER = 1,10
            NPHASE(IMPER) = 0
   54     continue

c --      read and store
          call rdainr ('imper_4e2',1,nmper,imper_4e2,100,ifnd)
          call rdfinr ('impphase',1,nmper,impphase,100,ifnd)
          call rdfdor ('wlsman',(altcu-500.0d0),altcu,wlsman_t,100,ifnd)
          call rdfdor ('gwlcrit',-500.0d0,0.0d0,gwlcrit_t,100,ifnd)
          call rdfdor ('hcrit',-1000.0d0,0.0d0,hcrit_t,100,ifnd)
          call rdfdor ('vcrit',0.0d0,20.0d0,vcrit_t,100,ifnd)

c ---     convert
          do i = 1, ifnd
            wlsman(imper_4e2(i),impphase(i)) = wlsman_t(i) - Altcu
            gwlcrit(imper_4e2(i),impphase(i)) = gwlcrit_t(i)
            hcrit(imper_4e2(i),impphase(i)) = hcrit_t(i)
            vcrit(imper_4e2(i),impphase(i)) = vcrit_t(i)
          enddo

c ---     check values
          do i = 1, ifnd
            imper = imper_4e2(i)
            iph = impphase(i)

            nphase(imper) = nphase(imper) + 1
            if (wlsman(imper,iph).lt.zbotdr(1+nrpri)) then
              write(6,*) 'rddre - level of automatic weir below bottom'
              write(6,*) 'of deepest channel of secondary system'
              stop
            endif
            if (swman(imper) .ne. 2) then
              write(*,*) 'rddre: #4e swman - imper mismatch'
              stop 
            endif
            if (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            endif
          enddo

          if (imperi .ne. nrman2) then
            write(*,*) 'rddre: #4e number of periods incorrect'
            stop
          endif

C ---     consistency checks WLSMAN, GWLCRIT, HCRIT and VCRIT
          DO 446 imper = 1,nmper
            IF (SWMAN(imper).EQ.2) THEN
              if (abs(gwlcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - GWLCRIT(1) must be 0.'
              if (abs(vcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - VCRIT(1) must be 0.'
              if (abs(hcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - HCRIT(1) must be 0.'

              if (swman(imper).eq.2 .and. 
     &          hbweir(imper).gt. (wlsman(imper,1)-0.99999d0)) then
               STOP 'RDDRE: #4e - HBWEIR within 1 cm of wlsman(1)' 
              endif

              DO 448 iph = 2,nphase(imper)
                if (wlsman(imper,iph).LT.wlsman(imper,iph-1))
     &          STOP 'RDDRE: #4e - WLSMAN inconsistent'
                if (GWLCRIT(imper,iph).GT.GWLCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - GWLCRIT inconsistent'
                if (HCRIT(imper,iph).GT.HCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - HCRIT inconsistent'
                if (VCRIT(imper,iph).LT.VCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - VCRIT inconsistent'
448           CONTINUE
            ENDIF 
446       CONTINUE
        ENDIF
      ENDIF
C ----------------------------------------------------------------------
C --- initialize counter for number of target level adjustments
      numadj = 0

C --- sttab(i,1) contains depths

C --- sw-levels, to be used in piece-wise linear functions (first level)
C --- is 100 cm above soil surface to allow for situations with ponding)
      sttab(1,1) = 100.0d0
      sttab(2,1) =   0.0
      do 100 i = 3,22
C ---   layer between surface level and deepest
C ---   drain/channel bottom is divided into 20 compartments
        sttab(i,1) = zbotdr(1+nrpri)*(i-2)/20.0
100   continue

C --- sttab(i,2) contains storage expressed in cm

C --- calculation of surface water storage (in cm, i.e. volume
C --- per unit area), as a function of sw-level, only for open channels:
      do 104 i = 1,22
        sttab(i,2) = 0.0
        do 108 ilev = 1+nrpri,nrsrf
          if (swdtyp(ilev).eq.0.and.sttab(i,1).gt.zbotdr(ilev)) then
C ---       for levels above soil surface the volume-increment is
C ---       computed for a rectangle, and not a trapezium
            if (sttab(i,1) .le. 0.0) then
              wdepth   = sttab(i,1)-zbotdr(ilev)
              wvolum   = wdepth*(widthr(ilev)+wdepth/taludr(ilev))
            else
              wdepth   = -zbotdr(ilev)
              wvolum   = wdepth*(widthr(ilev)+wdepth/taludr(ilev))
              wbreadth = widthr(ilev) + 2*wdepth/taludr(ilev)
              wdepth   = sttab(i,1)
              wvolum   = wvolum + wbreadth*wdepth
            endif
            sttab(i,2) = sttab(i,2)+wvolum/l(ilev)
          endif
108     continue
104   continue

C --- initial storage wstini
      wstini = wstlev (wls1,sttab)
      wst = wstini
	if (GetErrorStatus().ne.0) return
C --- initialize memorization of wls for most recent 4 timesteps 
      do 200 i=1,4
        wlsbak(i) = 0.0
 200  continue

C --- close input file with lateral boundary conditions
      CLOSE (DRE)         

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDGRASS (CRPFIL,ENVSTR,TDWI,LAIEM,RGRLAI,SLATB,
     &  SSA,SPAN,TBASE,KDIF,KDIR,EFF,AMAXTB,TMPFTB,TMNFTB,CVL,CVR,CVS,
     &  Q10,RML,RMR,RMS,RFSETB,FRTB,FLTB,FSTB,PERDL,RDRRTB,
     &  RDRSTB,HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,
     &  ADCRL,ECMAX,ECSLOP,COFAB,RDI,RRI,RDC,RDCTB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 6/6/01   
C     Purpose            : get crop parameters for grass     
C     Subroutines called : 
C     Functions called   : AFGEN
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER CRP,I,LOGF,IFND
      integer*4 getun2
 
      REAL*8 SLATB(30),AMAXTB(30),TMPFTB(30)
      REAL*8 TMNFTB(30),RFSETB(30),FRTB(30),FLTB(30),FSTB(30),RDRRTB(30)
      REAL*8 RDRSTB(30),KDIF,KDIR,LAIEM,ECMAX,ECSLOP,COFAB,RSC
      REAL*8 HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RDCTB(22)
      REAL*8 X,SUM,RFAC,VALUE,ADCRH,ADCRL,AFGEN,CVL,CVR,CVS,EFF,RMR
      REAL*8 PERDL,Q10,RDC,RDI,RGRLAI,RML,RMS,RRI,SPAN,SSA,TBASE,TDWI

      CHARACTER CRPFIL*16,ENVSTR*50,FILNAM*80


C ----------------------------------------------------------------------
C --- initialise and start reading
      filnam = trim(envstr)//trim(crpfil)//'.crp'
      crp = getun2 (10,19,2)
      call rdinit(crp,logf,filnam)

C --- initial
      call rdsdor ('tdwi',0.0d0,10000.0d0,tdwi)
      call rdsdor ('laiem',0.0d0,10.0d0,laiem)
      call rdsdor ('rgrlai',0.0d0,1.0d0,rgrlai)

C --- green area
      call rdador ('slatb',0.0d0,366.0d0,slatb,30,ifnd)
      call rdsdor ('ssa',0.0d0,1.0d0,ssa)
      call rdsdor ('span',0.0d0,366.0d0,span)
      call rdsdor ('tbase',-10.0d0,30.0d0,tbase)

C --- assimilation
      call rdsdor ('kdif',0.0d0,2.0d0,kdif)
      call rdsdor ('kdir',0.0d0,2.0d0,kdir)
      call rdsdor ('eff',0.0d0,10.0d0,eff)
      call rdador ('amaxtb',0.0d0,366.0d0,amaxtb,30,ifnd)
      call rdador ('tmpftb',-10.0d0,50.0d0,tmpftb,30,ifnd)
      call rdador ('tmnftb',-10.0d0,50.0d0,tmnftb,30,ifnd)

C --- conversion of assimilates into biomass
      call rdsdor ('cvl',0.0d0,1.0d0,cvl)
      call rdsdor ('cvr',0.0d0,1.0d0,cvr)
      call rdsdor ('cvs',0.0d0,1.0d0,cvs)

C --- maintenance respiration
      call rdsdor ('q10',0.0d0,5.0d0,q10)
      call rdsdor ('rml',0.0d0,1.0d0,rml)
      call rdsdor ('rmr',0.0d0,1.0d0,rmr)
      call rdsdor ('rms',0.0d0,1.0d0,rms)
      call rdador ('rfsetb',0.0d0,366.0d0,rfsetb,30,ifnd)

C --- partitioning
      call rdador ('frtb',0.0d0,366.0d0,frtb,30,ifnd)
      call rdador ('fltb',0.0d0,366.0d0,fltb,30,ifnd)
      call rdador ('fstb',0.0d0,366.0d0,fstb,30,ifnd)

C --- death rates
      call rdsdor ('perdl',0.0d0,3.0d0,perdl)
      call rdador ('rdrrtb',0.0d0,366.0d0,rdrrtb,30,ifnd)
      call rdador ('rdrstb',0.0d0,366.0d0,rdrstb,30,ifnd)

C --- water use
      call rdsdor ('hlim1' ,-100.0d0,100.0d0,hlim1)
      call rdsdor ('hlim2u',-1000.0d0,100.0d0,hlim2u)
      call rdsdor ('hlim2l',-1000.0d0,100.0d0,hlim2l)
      call rdsdor ('hlim3h',-10000.0d0,100.0d0,hlim3h)
      call rdsdor ('hlim3l',-10000.0d0,100.0d0,hlim3l)
      call rdsdor ('hlim4' ,-16000.0d0,100.0d0,hlim4)
      call rdsdor ('rsc',0.0d0,1000.0d0,rsc)
      call rdsdor ('adcrh',0.0d0,5.0d0,adcrh)
      call rdsdor ('adcrl',0.0d0,5.0d0,adcrl)
      call rdsdor ('ecmax',0.0d0,20.0d0,ecmax)
      call rdsdor ('ecslop',0.0d0,40.0d0,ecslop)
      call rdsdor ('cofab',0.0d0,1.0d0,cofab)

C --- rooting
      call rdador ('rdctb',0.0d0,100.0d0,rdctb,22,ifnd)
      call rdsdor ('rdi',0.0d0,1000.0d0,rdi)
      call rdsdor ('rri',0.0d0,100.0d0,rri)
      call rdsdor ('rdc',0.0d0,1000.0d0,rdc)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 I = 5,100,10
        X = 0.01d0*dble(I)
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
96    CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
        RDCTB(I*2) = RDCTB(I*2)*RFAC
   98 continue

      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDHEA (PRJNAM,ENVSTR,LOGF,SWSHF,TAMPLI,TMEAN,DDAMP,
     &  TIMREF,TEMPI,NUMNOD)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                                    
C     Purpose            : read soil heat transport input data     
C                          
C     Subroutines called : SHIFTL,,getun                                         
C     Functions called   : -                     
C     File usage         :                                     
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER   LOGF,SWSHF,NUMNOD
      integer*4 getun2

      REAL*8    TAMPLI,TMEAN,DDAMP,TIMREF,TEMPI(MACP)

      CHARACTER PRJNAM*16,ENVSTR*50 ,FILNAM*80
C-----------------------------------------------------------------------
C --- local
      INTEGER   HEA,CP
      REAL*8    TEMP(MACP)

C ----------------------------------------------------------------------

C --- open and read input file
      filnam = trim(envstr)//trim(prjnam)//'.hea'
      hea = getun2 (10,19,2)
      call rdinit(hea,logf,filnam)

C --- method 
      call rdsinr ('swshf',1,2,swshf)

      if (swshf.eq.1) then
        call rdsdor ('tampli',0.0d0,50.0d0,tampli)
        call rdsdor ('tmean',5.0d0,30.0d0,tmean)
        call rdsdor ('ddamp',0.0d0,500.0d0,ddamp)
        call rdsdor ('timref',0.0d0,366.0d0,timref)
      elseif (swshf.eq.2) then
        call rdfdor ('tempi',-20.0d0,50.0d0,temp,macp,numnod)
        do 30  cp = 1,numnod
          tempi(cp) = temp(cp)
   30   continue
      endif

      close(hea)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE RDKEY (PRJNAM,prjdes,ENVSTR,BRUNY,BRUND,ERUNY,
     &  ERUND,ISEQ,FMAY,PERIOD,SWRES,OUTD,OUTY,METFIL,LAT,ALT,SWETR,
     &  SWRAI,IRGFIL,CALFIL,DRFIL,BBCFIL,OUTFIL,NOFRNS,
     &  FLCAL,SWDRA,SWSOLU,SWHEA,SWVAP,SWDRF,SWSWB,SWAFO,AFONAM,
     &  SWAUN,AUNNAM,SWATE,ATENAM,SWAIR,AIRNAM,SWSCRE,NOUTDA,sw1fil)
C ----------------------------------------------------------------------
C     Date               : 6/6/01   
C     Purpose            : read general input data SWAP
C     Subroutines called : ,DATONR 
C     Functions called   : FROMTO
C     File usage         : SWAP.KEY, file with general input data
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi' 

      INTEGER BRUND,BRUNY,ERUND,ERUNY,ISEQ,FMAY,PERIOD,SWRES
      INTEGER OUTD(1000),OUTY(1000),SWETR,SWRAI,NOFRNS,SWDRA,SWSOLU
      INTEGER SWHEA,SWVAP,SWDRF,SWSWB,SWAFO,SWAUN,SWATE,SWAIR,SWSCRE
	INTEGER NOUTDA,sw1fil

      REAL*8  LAT,ALT
      
      CHARACTER   PRJNAM*16,ENVSTR*50,METFIL*15
      CHARACTER*16 IRGFIL(MAYRS),CALFIL(MAYRS),DRFIL(MAYRS)
      CHARACTER*16 BBCFIL(MAYRS),OUTFIL(MAYRS)
      CHARACTER*16 AFONAM,AUNNAM,ATENAM,AIRNAM
      character*80 prjdes

      LOGICAL FLCAL,EXISTS

C ----------------------------------------------------------------------
C --- local
      INTEGER DD(1000),MM(1000),D1,M1,Y1,KEY
      INTEGER FROMTO,I,NR,SWODAT,FDAY,NextAYr
	INTEGER*4   LOGF
	LOGICAL     ToLog, ToScr

      CHARACTER keyfil*80, filnam*80, line*80

      integer*4 datea(6), getun2
	integer*2 Posarg, Numchar
      real*8    ssrun, esrun, outdat(1000)
      real*4    fsec
C ----------------------------------------------------------------------

c --- Path and filename of executable through argument command line
      PosArg = 1
      Call GetArg (PosArg,keyfil,NumChar)
      if (Numchar.lt.1) keyfil = 'swap'
      prjdes = keyfil
      filnam = trim(keyfil)//'.swp'

C --  write head to logfile
	call MessInq (ToScr, ToLog, logf)
	if (.not.ToLog) call FatalERR ('WriteHead','no logfile active')

      Call WriteHead(logf,1,prjdes)
      write (line,15)  
   15 format('*',70('-'),'*')
      write(logf,*)
	call messwrt (line,' ')
	call messwrt ('I/O messages',' ')
	
c --- open file
      key = getun2 (10,19,2)
      call rdinit(key,0,filnam)

C -   environment
      call rdscha ('project',prjnam)
      call rdscha ('path',envstr)
      call rdsinr ('swscre',0,2,swscre)

C -   time variables
      call rdstim('ssrun',ssrun)
      call dtdpar (ssrun, datea, fsec)
      bruny = datea(1)
      call datonr (bruny,datea(2),datea(3),brund)
      call rdstim('esrun',esrun)
      call dtdpar (esrun, datea, fsec)
      eruny = datea(1)
      call datonr (eruny,datea(2),datea(3),erund)
      call rdsinr ('fmay',1,12,fmay)
      call rdsinr ('period',0,366,period)
      call rdsinr ('swres',0,1,swres)
      call datonr (bruny,fmay,1,fday)
      nextayr = bruny
      if (brund.ge.fday) nextayr = bruny+1
      iseq = 0
      if (fromto(brund,bruny,erund,eruny).ge.
     &    fromto(brund,bruny,fday,nextayr)) iseq = 1

C -   read additional output dates
      do 4 i = 1,1000
        outd(i) = 0
        outy(i) = 0
4     continue
      call rdsinr ('swodat',0,1,swodat)
      if (swodat.eq.1) then
        call rdatim ('outdat',outdat,1000,noutda)
      end if
      do i = 1,noutda
        call dtdpar(outdat(i),datea,fsec)
        call datonr (datea(1),datea(2),datea(3),outd(i))
        outy(i) = datea(1)
      enddo
      if (period.eq.0 .and. noutda.eq.0) then
         print *,'no output dates specified'
         stop
      endif

C -   meteo
      call rdscha ('metfil',metfil)
      call rdsdor ('lat',-60.0d0,60.0d0,lat)
      call rdsdor ('alt',-400.0d0,3000.0d0,alt)
      call rdsinr ('swetr',0,1,swetr)
      call rdsinr ('swrai',0,1,swrai)

C -   rerun list
      call rdsinr ('sw1fil',0,1,sw1fil)
      call rdacha ('irgfil',irgfil,mayrs,nofrns)
      call rdfcha ('calfil',calfil,mayrs,nofrns)
      call rdfcha ('drfil',drfil,mayrs,nofrns)
      call rdfcha ('bbcfil',bbcfil,mayrs,nofrns)
      call rdfcha ('outfil',outfil,mayrs,nofrns)
      flcal = .true.
c     perform simpel check on file names of sub-run list
      if (sw1fil.eq.1) then
        do i = 1, nofrns
          if (irgfil(i).ne.irgfil(1)) then
            write(6,11) trim(irgfil(i))
            write(logf,11) trim(irgfil(i))
   11       format(/' fatal error in key-file: list of sub-run',
     &              ' filenames inconsistent: ',a)
            stop
          endif
          if (bbcfil(i).ne.bbcfil(1)) then
            write(6,11) trim(bbcfil(i))
            write(logf,11) trim(bbcfil(i))
            stop
          endif
          if (outfil(i).ne.outfil(1)) then
            write(6,11) trim(outfil(i))
            write(logf,11) trim(outfil(i))
            stop
          endif
        enddo
      endif

C -   drainage
      call rdsinr ('swdra',0,2,swdra)

C -   solute
      call rdsinr ('swsolu',0,1,swsolu)

C -   heat
      call rdsinr ('swhea',0,1,swhea)

C -   summary
      call rdsinr ('swvap',0,1,swvap)

C -   drainage fluxes, surface reservoir 
      if (swdra.eq.2) then
        call rdsinr ('swdrf',0,1,swdrf)
        call rdsinr ('swswb',0,1,swswb)
      endif
 
C -   output for water quality models
      call rdsinr ('swafo',0,2,swafo)
      if (swafo.ge.1) call rdscha ('afonam',afonam)

      call rdsinr ('swaun',0,2,swaun)
      if (swaun.ge.1) call rdscha ('aunnam',aunnam)

      call rdsinr ('swate',0,1,swate)
      if (swate.eq.1) call rdscha ('atenam',atenam)

      call rdsinr ('swair',0,1,swair)
      if (swair.eq.1) call rdscha ('airnam',airnam)

      close (key)


C --- Copy content of key-file to log-file
      write (logf,14)  
   14 format('*',70('-'),'*',/,' Echo of input file:',/)
      call COPFL2(key,filnam,logf,.true.)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE RDSOL (SOLFIL,ENVSTR,LAY,FLGENU,THETHI,THETLO,HTABLE,
     & KTABLE,THETSL,KSAT,DMCH,DMCC,COFGEN,SWSCAL,ISCLAY,FSCALE,IRUNNR,
     & KSE99,LOGF,SWHYST)
C ----------------------------------------------------------------------
C     Date               : 6/6/01
C     Purpose            : reading the soil physical data              
C     Subroutines called :           
C     File usage         : SOLFIL, file containing soil characteristics        
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'
 
      INTEGER   LAY,SWSCAL,ISCLAY,THETLO(MAHO),THETHI(MAHO)
      INTEGER   IRUNNR,LOGF,SWHYST

      REAL*8    fscale(MAYRS)

      REAL*8    HTABLE(MAHO,99),COFGEN(8,MAHO),KTABLE(MAHO,99)
      REAL*8    DMCC(MAHO,99),DMCH(MAHO,99),THETSL(MAHO),KSAT(MAHO)
      REAL*8    KSE99(MAHO)

      CHARACTER SOLFIL*16,ENVSTR*50

      LOGICAL   FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER   sol,swphys,i,j,ifnd
      integer*4 getun2
      REAL*8    relsat
      REAL*8    exponl,term1,THETB(99),HBUF(99),KBUF(99)
      CHARACTER filnam*80

C ----------------------------------------------------------------------
C --- initialise and start reading
      filnam = trim(envstr)//trim(solfil)//'.sol'
      sol = getun2 (10,19,2)
      call rdinit(sol,logf,filnam)

C --- method to describe soil physical parameters
      CALL RdSINR ('SWPHYS',0,1,SWPHYS)
      IF (SWPHYS.EQ.0) THEN
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,2) FILNAM
	    WRITE(*,2) FILNAM
 2        FORMAT(' Error in file:',A,': In order to calculate ',
     &     'hysteresis, the soil hydraulic functions',/,
     &     ' should be described by the Mualem-Van Genuchten model;',/,
     &     ' please adapt the *.SOL file')
          write(*,*) 'Check log file'
          STOP 
	  ENDIF
        FLGENU(LAY) = .FALSE.
      ELSE
        FLGENU(LAY) = .TRUE.
      ENDIF

C --- soil physical data presented as a table
      IF (.NOT.FLGENU(LAY)) THEN
C ---   read table
        call rdador ('thetb',0.0d0,1.0d0,thetb,99,ifnd)
        call rdfdor ('hbuf',-1.0d10,0.0d0,hbuf,99,ifnd)
        call rdfdor ('kbuf',0.0d0,1.0d2,kbuf,99,ifnd)

        THETLO(LAY) = INT(100.0D0 *THETB(1)+0.5D0)
        THETHI(LAY) = INT(100.0D0 * THETB(IFND) + 0.5D0)

C ---   saturated water content and hydraulic conductivity
        THETSL(LAY) = THETB(IFND)
        KSAT(LAY)   = KBUF(IFND)

C ---   fill HTABLE and KTABLE
        J = 1
        DO 14 I = THETLO(LAY),THETHI(LAY)
          HTABLE(LAY,I) = HBUF(J)
          KTABLE(LAY,I) = KBUF(J)
          J = J+1
14      CONTINUE

C ---   check pressure head range
        IF (HTABLE(LAY,THETLO(LAY)) .GT. -0.99D6) THEN
          WRITE(LOGF,16) FILNAM
          WRITE(*,16) FILNAM
 16       FORMAT(' Fatal error in file: ',A,
     &           ': the lowest pressure head in',/, 
     &           ' the table of the soil hydraulic functions should be',
     &           ' smaller than or equal to -1.0e6')
          write(*,*) 'Check log file'
          STOP 
        ENDIF

        IF (HTABLE(LAY,THETHI(LAY)) .LT. -1.0D-6) THEN
          WRITE(LOGF,18) FILNAM
          WRITE(*,18) FILNAM
 18       FORMAT(' Fatal error in file: ',A,
     &           ': the highest pressure head in',/,
     &           ' the table of the soil hydraulic functions should be',
     &           ' equal to zero')
          write(*,*) 'Check log file'
          STOP 
        ENDIF

C ---   prepare dmc-table
        DO 20 I = THETLO(LAY),THETHI(LAY)-1
          DMCH(LAY,I) = 0.5D0*(HTABLE(LAY,I)+HTABLE(LAY,I+1))
          DMCC(LAY,I) = 0.01D0/(HTABLE(LAY,I+1)-HTABLE(LAY,I))
20      CONTINUE
        DMCH(LAY,THETHI(LAY)) = 0.0D0
        DMCC(LAY,THETHI(LAY)) = DMCC(LAY,THETHI(LAY)-1)

        HTABLE(LAY,THETHI(LAY)+1) = -1.0D0*HTABLE(LAY,THETHI(LAY)-1)
        HTABLE(LAY,THETHI(LAY)+2) = -1.0D0*HTABLE(LAY,THETHI(LAY)-2) 
        KTABLE(LAY,THETHI(LAY)+1) = KTABLE(LAY,THETHI(LAY))
        KTABLE(LAY,THETHI(LAY)+2) = KTABLE(LAY,THETHI(LAY))
      ENDIF

C --- soil physical data according to Van Genuchten
      if (flgenu(lay)) then
        call rdsdor ('cofgen1',0.0d0,0.4d0,cofgen(1,lay))
        call rdsdor ('cofgen2',dble(cofgen(1,lay)),0.95d0,cofgen(2,lay))
        call rdsdor ('cofgen3',1.0d-2,1.0d4,cofgen(3,lay))
        call rdsdor ('cofgen4',0.1d-4,1.0d0,cofgen(4,lay))
        call rdsdor ('cofgen5',-25.0d0,25.0d0,cofgen(5,lay))
        call rdsdor ('cofgen6',1.0d0,5.0d0,cofgen(6,lay))
        if (swhyst .ne. 0) then
          call rdsdor('cofgen8',dble(cofgen(4,lay)),1.0d0,cofgen(8,lay))
	  endif
        cofgen(7,lay) = 1.0d0 - (1.0d0/cofgen(6,lay))
        cofgen(5,lay) = cofgen(7,lay) * (cofgen(5,lay)+2.0d0)

C --- modify some parameters according to scaling factors
        if (swscal.eq.1.and.isclay.eq.lay) then
          cofgen(3,lay) = cofgen(3,lay)*fscale(irunnr)**2
          cofgen(4,lay) = cofgen(4,lay)*fscale(irunnr)
        endif

C --- saturated water content and conductivity for each layer
        thetsl(lay) = cofgen(2,lay)
        ksat(lay)   = cofgen(3,lay)
        relsat      = 0.99d0
        exponl      = cofgen(5,lay)/cofgen(7,lay)-2.0d0
        term1       = (1.0d0-relsat**(1.0/cofgen(7,lay)))**cofgen(7,lay)
        kse99(lay)  = cofgen(3,lay)*(relsat**exponl)*(1.0d0-term1)*
     &                (1.0d0-term1)
      ENDIF

      CLOSE (SOL)
      RETURN
      END


C ----------------------------------------------------------------------
      SUBROUTINE RDSLT (PRJNAM,ENVSTR,LOGF,CPRE,CMLI,DDIF,LDIS,TSCF,KF,
     &  FREXP,CREF,DECPOT,GAMPAR,RTHETA,BEXP,FDEPTH,KMOBIL,
     &  HAQUIF,POROS,KFSAT,DECSAT,CDRAIN,NUMNOD,NUMLAY,SWBR)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                                     
C     Purpose            : read soil water and profile characteristics     
C                          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER NUMNOD,NUMLAY,LOGF,SWBR
      integer*4 getun2

      REAL*8 CPRE,CMLI(MACP),DDIF,LDIS,TSCF,KF,FREXP,CREF 
      REAL*8 DECPOT,GAMPAR,RTHETA,BEXP,FDEPTH(MAHO),KMOBIL,HAQUIF,POROS
      REAL*8 KFSAT,DECSAT,CDRAIN

      CHARACTER PRJNAM*16,ENVSTR*50,FILNAM*80
C ----------------------------------------------------------------------
C --- local
      INTEGER SLT,LAY,SWSP,SWDC,SWPREF

      LOGICAL EXISTS
C ----------------------------------------------------------------------

C --- open and read file
      filnam = trim(envstr)//trim(prjnam)//'.slt'
      slt = getun2 (10,19,2)
      call rdinit(slt,logf,filnam)

      call rdsdor ('cpre',0.0d0,100.0d0,cpre)
      call rdfdor ('cmli',0.0d0,1000.0d0,cmli,macp,numnod)
      call rdsdor ('ddif',0.0d0,10.0d0,ddif)
      call rdsdor ('ldis',0.0d0,100.0d0,ldis)
      call rdsdor ('tscf',0.0d0,10.0d0,tscf)

C --- sorption
      call rdsinr ('swsp',0,1,swsp)
      if (swsp .eq. 1) then
        call rdsdor ('kf',0.0d0,100.0d0,kf)
        call rdsdor ('frexp',0.0d0,10.0d0,frexp)
        call rdsdor ('cref',0.0d0,1000.0d0,cref)
      else if (swsp.eq.0) then
        kf    = 0.0d0
        frexp = 1.0d0
        cref  = 1.0d0
      endif

C --- decomposition
      call rdsinr ('swdc',0,1,swdc)
      if (swdc .eq. 1) then
        call rdsdor ('decpot',0.0d0,10.0d0,decpot)
        call rdsdor ('gampar',0.0d0,0.5d0,gampar)
        call rdsdor ('rtheta',0.0d0,0.4d0,rtheta)
        call rdsdor ('bexp',0.0d0,2.0d0,bexp)
        call rdfdor ('fdepth',0.0d0,1.0d0,fdepth,maho,numlay)
      else if (swdc .eq. 0) then
        decpot = 0.0d0
        gampar = 0.0d0
        rtheta = 0.5d0
        bexp   = 0.0d0
        do 181 lay = 1,maho
          fdepth(lay) = 0.0d0
  181   continue
      endif

C --- preferential flow 
      call rdsinr ('swpref',0,1,swpref)
      if (swpref .eq. 1) then
        call rdsdor ('kmobil',0.0d0,100.0d0,kmobil)
      elseif (swpref .eq. 0) then
        kmobil = 0.0d0
      endif

C --- breakthrough
      call rdsinr ('swbr',0,1,swbr)
      if (swbr .eq. 0) then
        call rdsdor ('cdrain',0.0d0,100.0d0,cdrain)
        haquif = 100.0d0
        poros  = 1.0d0
        kfsat  = 0.0d0
        decsat = 0.0d0
      elseif (swbr .eq. 1) then
        call rdsdor ('haquif',0.0d0,10000.0d0,haquif)
        call rdsdor ('poros',0.0d0,0.6d0,poros)
        call rdsdor ('kfsat',0.0d0,100.0d0,kfsat)
        call rdsdor ('decsat',0.0d0,10.0d0,decsat)
        call rdsdor ('cdraini',0.0d0,100.0d0,cdrain)
      endif

      close(slt)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE RDSWA(PRJNAM,ENVSTR,LOGF,PONDMX,SWREDU,COFRED,RSIGNI,
     &  DTMIN,DTMAX,THETOL,NUMLAY,NUMNOD,BOTCOM,DZ,SOLFIL,RDS,
     &  SWHYST,TAU,SWSCAL,NSCALE,ISCLAY,FSCALE,SWMOBI,PF1,FM1,PF2,FM2,
     &  THETIM,SWCRACK,SHRINA,MOISR1,MOISRD,ZNCRACK,GEOMF,DIAMPOL,
     &  RAPCOEF,DIFDES,SWDIVD,COFANI,SWINCO,HI,GWLI,THETCR,PSAND,PSILT,
     &  PCLAY,ORGMAT,CFBS,SWCFBS,SWBALANCE,MSTEPS,GWLCONV)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                               
C     Purpose            : read soil water and profile characteristics     
C                          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER LOGF,SWREDU,NUMLAY,NUMNOD,BOTCOM(MAHO),SWHYST
      INTEGER SWSCAL,NSCALE,ISCLAY,SWMOBI,SWCRACK,SWDIVD,SWINCO,SWCFBS
	INTEGER SWBALANCE,MSTEPS

      REAL*8 PONDMX,COFRED,RSIGNI,DTMIN,DTMAX,THETOL,DZ(MACP),RDS
      REAL*8 FSCALE(MAYRS),PF1(MAHO),FM1(MAHO),PF2(MAHO),FM2(MAHO)
      REAL*8 MOISR1,MOISRD,ZNCRACK,GEOMF,DIAMPOL,RAPCOEF,DIFDES
      REAL*8 COFANI(MAHO),GWLI,THETCR(MAHO),PSAND(MAHO),PSILT(MAHO)
	REAL*8 PCLAY(MAHO),ORGMAT(MAHO),CFBS,SHRINA
      REAL*8 TAU,THETIM(MAHO),HI(MACP),GWLCONV

      CHARACTER   PRJNAM*16,ENVSTR*50
      CHARACTER*16 SOLFIL(MAHO)

C ----------------------------------------------------------------------
C --- local variables
      INTEGER SWA,IFND,LAY
      integer*4 getun2
      CHARACTER   FILNAM*80

C ----------------------------------------------------------------------

C --- open file with soil parameters
      filnam = trim(envstr)//trim(prjnam)//'.swa'
      swa = getun2 (10,19,2)
      call rdinit(swa,logf,filnam)

C --- ponding
      call rdsdor ('pondmx',0.0d0,1000.0d0,pondmx)

C --- soil evaporation
      call rdsinr ('swcfbs',0,1,swcfbs)
      if (swcfbs.eq.1) then
        call rdsdor ('cfbs',0.5d0,1.5d0,cfbs)
      else
        cfbs = 1.0d0
	endif
      call rdsinr ('swredu',0,2,swredu)
      if (swredu .gt.0) then
        call rdsdor ('cofred',0.0d0,1.0d0,cofred)
        call rdsdor ('rsigni',0.0d0,1.0d0,rsigni)  
      endif
 
C --- parameters numerical scheme
      call rdsdor ('dtmin', 1.0d-8, 1.0d-1,dtmin)
      call rdsdor ('dtmax', 1.0d-2, 5.0d-1,dtmax)
      call rdsdor ('thetol',1.0d-5, 1.0d-2,thetol)
      call rdsdor ('gwlconv',1.0d-5,1000.0d0,gwlconv)
      call rdsinr ('msteps',2,100000000,msteps)
      call rdsinr ('swbalance',0,1,swbalance)

C --- geometry of the soil profile
      call rdsinr ('numlay',1,maho,numlay)
      call rdsinr ('numnod',1,macp,numnod)
      call rdfinr ('botcom',1,numnod,botcom,maho,numlay)

C --- thickness of compartments
      call rdfdor ('dz',1.0d-6,5.0d2,dz,macp,numnod)

C --- files with soil hydraulic functions
      call rdfcha ('solfil',solfil,maho,numlay)

C --- soil texture and organic matter per soil layer
      call rdfdor ('psand',0.0d0,1.0d0,psand,maho,numlay)
      call rdfdor ('psilt',0.0d0,1.0d0,psilt,maho,numlay)
      call rdfdor ('pclay',0.0d0,1.0d0,pclay,maho,numlay)
      call rdfdor ('orgmat',0.0d0,1.0d0,orgmat,maho,numlay)

C --- rooting depth limitation
      call rdsdor ('rds',1.0d0,1000.0d0,rds)

C --- hysteresis
      call rdsinr ('swhyst',0,2,swhyst)
      if (swhyst.gt.0) then
        call rdsdor ('tau',0.0d0,1.0d0,tau)
      endif

C --- scaling
      call rdsinr ('swscal',0,1,swscal)
      IF (SWSCAL.EQ.1) then
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,40) FILNAM
	    WRITE(*,40) FILNAM
 40       FORMAT(' Hysteresis in combination with scaling is not',
     &           ' allowed; adapt: ',A)
          write(*,*) 'Check log file'
          STOP
	  ENDIF
        call rdsinr ('nscale',1,mayrs,nscale)
        call rdsinr ('isclay',1,maho,isclay)
        call rdador ('fscale',0.0d0,100.0d0,fscale,mayrs,ifnd)
      ENDIF

C --- preferential flow due to immobile water
      call rdsinr ('swmobi',0,1,swmobi)
      IF (SWMOBI.EQ.1) THEN
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,42) FILNAM
	    WRITE(*,42) FILNAM 
 42       FORMAT(' Hysteresis in combination with preferential',
     &           ' flow is not allowed; adapt: ',A)
          write(*,*) 'Check log file'
          STOP
	  ENDIF
	  IF (SWSCAL.EQ.1) THEN
          WRITE(LOGF,44) FILNAM
	    WRITE(*,44) FILNAM
 44       FORMAT(' Scaling in combination with preferential',
     &           ' flow is not allowed; adapt: ',A)
          write(*,*) 'Check log file'
          STOP
	  ENDIF
C       read table
        call rdfdor ('pf1',0.0d0,5.0d0,pf1,maho,numlay)
        call rdfdor ('fm1',0.0d0,1.0d0,fm1,maho,numlay)
        call rdfdor ('pf2',0.0d0,5.0d0,pf2,maho,numlay)
        call rdfdor ('fm2',0.0d0,1.0d0,fm2,maho,numlay)
        call rdfdor ('thetim',0.0d0,1.0d0,thetim,maho,numlay)
      else if (swmobi.eq.0) then
        do 20  lay = 1, maho
          pf1(lay) = 0.0
          fm1(lay) = 0.0
          pf2(lay) = 0.0
          fm2(lay) = 0.0
          thetim(lay) = 0.0
   20   continue
      endif

C --- flow through soil cracks
      call rdsinr ('swcrack',0,1,swcrack)
      if (swcrack.eq.1) then
        call rdsdor ('shrina',0.0d0,2.0d0,shrina)
        call rdsdor ('moisr1',0.0d0,5.0d0,moisr1)
        call rdsdor ('moisrd',0.0d0,1.0d0,moisrd)
        call rdsdor ('zncrack',-100.0d0,0.0d0,zncrack)
        call rdsdor ('geomf',0.0d0,100.0d0,geomf)
        call rdsdor ('diampol',0.0d0,100.0d0,diampol)
        call rdsdor ('rapcoef',0.0d0,10000.0d0,rapcoef)
        call rdsdor ('difdes',0.0d0,10000.0d0,difdes)
        call rdfdor ('thetcr',0.0d0,1.0d0,thetcr,maho,numlay)
      elseif (swcrack.eq.0) then
        shrina = 0.0
        moisr1 = 0.0
        moisrd = 0.0
        zncrack = 0.0
        geomf = 0.0
        diampol = 0.0
        rapcoef = 0.0
        difdes = 0.0
      end if

C --- fate of discharge water in a regional system
      call rdsinr ('swdivd',0,1,swdivd)
      if (swdivd.eq.1) then
        call rdfdor ('cofani',0.0d0,1000.0d0,cofani,maho,numlay)
      endif

C --- initial water conditions
      call rdsinr ('swinco',1,2,swinco)
      if (swinco.eq.1) then
        call rdfdor ('hi',-1.0d10,1.0d4,hi,macp,numnod)
      elseif (swinco.eq.2) then
        call rdsdor ('gwli',-5000.0d0,100.0d0,gwli)
      endif

      close (swa)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE REDUCEVA (SWREDU,REVA,PEVA,DAYNR,DAYSTA,NRAI,
     &                     NIRD,RUNNR,ISEQ,COFRED,RSIGNI)
C ----------------------------------------------------------------------
C     Date               : 18/08/98                                         
C     Purpose            : calculates reduction of soil evaporation
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      INTEGER SWREDU,DAYNR,DAYSTA,RUNNR,ISEQ

      REAL*8  REVA,PEVA,NRAI,NIRD,COFRED, RSIGNI
C ----------------------------------------------------------------------
C --- local
      REAL*8  ldwet,spev,saev,saevm1

      SAVE
C ----------------------------------------------------------------------
      if (swredu.eq.0) then
C ---   no reduction ---
        reva = peva
      elseif (swredu.eq.1) then
C ---   reduction with Black model ---
        if (daynr.eq.daysta .and. (runnr.eq.1.or.iseq.eq.0)
     &      .or. (nrai+nird) .gt. RSIGNI) ldwet = 0.0
        ldwet = ldwet + 1.0d0
        reva  = COFRED * (sqrt(ldwet)-sqrt(ldwet-1.0d0))
        reva  = min(reva,peva)
      elseif (swredu.eq.2) then
C ---   reduction with Boesten and Stroosnijder model ---
        if (daynr.eq.daysta .and. (runnr.eq.1.or.iseq.eq.0)
     &      .or. (nrai+nird) .gt. RSIGNI) then
          spev = 0.0
          saev = 0.0
        endif
        if ((nrai+nird) .lt. peva) then
          spev = spev + (peva - (nrai+nird))
          saevm1 = saev
          if (spev .lt. COFRED**2) then
            saev = spev
          else
            saev = COFRED * sqrt(spev)
          endif
          reva = nrai + nird + saev - saevm1
        else
          reva = peva
          saev = max (0.0d0,saev - (nrai+nird-peva))
          if (saev .lt. (COFRED**2)) then
            spev = saev
          else
            spev = (saev/COFRED)**2
          endif
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE ROOTEX (NUMNOD,QROT,DRZ,DZ,PTRA,Z,HLIM3H,HLIM3L,HLIM4,
     & HLIM2U,H,CML,BOTCOM,HLIM2L,HLIM1,ADCRL,ADCRH,ATMDEM,
     & RDCTAB,RDM,SWSOLU,ECMAX,ECSLOP,LAYER,THETSL,THETA,QROSUM)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                                          
C     Purpose            : calculates the root extraction rates (as a
C                          function of press. head) for each nodal point
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,BOTCOM(MAHO),SWSOLU,LAYER(MACP)

      REAL*8  QROT(MACP),DRZ,DZ(MACP),PTRA,Z(MACP),CML(MACP),RDCTAB(22)
      REAL*8  HLIM3H,HLIM3L,HLIM4,HLIM2U,HLIM2L,ECMAX,ECSLOP,RDM
      REAL*8  ADCRH,ADCRL,ATMDEM,HLIM1,HLIM2,HLIM3,AFGEN,QROSUM

      REAL*8  H(MACP),THETA(MACP),THETSL(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER node,noddrz,lay

      REAL*8  depth,SMEAN,top,bot,mid,lam,lam1,lam2,lam3,alpsol,alpwat
      REAL*8  ECsat
C ----------------------------------------------------------------------
C --- clean rtex-array
      do 10 node = 1,numnod
        qrot(node) = 0.0
10    continue
      QROSUM = 0.0

C --- skip routine if there are no roots
      if (drz.gt.-1.d-3) goto 500

C --- determine lowest compartment containing roots
      depth = 0.0
      do 20 node = 1,numnod
        noddrz = node
        depth  = depth - dz(node)
        if (drz.ge.depth-1.0d-6) goto 30
20    continue

30    SMEAN = ptra/abs(drz)

C --- calculate root extraction of the compartments
      do 100 node = 1,noddrz
        top = abs(z(node) + 0.5*dz(node))
        mid = abs(z(node))
        bot = abs(z(node) - 0.5*dz(node))
        if (z(node)- 0.5*dz(node).lt.drz) bot = abs(drz)
        if (abs(drz).lt.rdm .and. noddrz .lt. 8) then
          qrot(node) = SMEAN*(bot-top) 
        else
          lam1 = AFGEN (RDCTAB,22,top/abs(drz))
          lam2 = AFGEN (RDCTAB,22,mid/abs(drz))
          lam3 = AFGEN (RDCTAB,22,bot/abs(drz))
          lam  = (lam1+lam2+lam3)/3
          qrot(node) = SMEAN*(bot-top)*lam
        endif 
100   continue

C --- calculating critical point hlim3 according to Feddes
      hlim3=hlim3h
      if (ATMDEM.ge.ADCRL) then
        if (ATMDEM.le.ADCRH) hlim3=hlim3h+((ADCRH-ATMDEM)/(ADCRH-ADCRL))
     &  *(hlim3l-hlim3h)
      else
        hlim3 = hlim3l
      endif

C --- calculation of transpiration reduction
      hlim2 = hlim2u
      do 200 node = 1,noddrz
        alpwat = 1.0d0
        alpsol = 1.0d0

C ---   reduction due to water stress
        if (node.gt.botcom(1)) hlim2=hlim2l
C ---   waterstress/ wet circumstances
        if (h(node).le.hlim1.and.h(node).gt.hlim2) 
     &    alpwat = (hlim1-h(node))/(hlim1-hlim2)
C ---   waterstress/ dry circumstances
        if (h(node).le.hlim3.and.h(node).ge.hlim4) 
     &    alpwat = (hlim4-h(node))/(hlim4-hlim3)
C ---   no transpiration 
        if (h(node).gt.hlim1.or.h(node).lt.hlim4)
     &    alpwat = 0.0

C ---   reduction due to salt stress
        if (SWSOLU.EQ.1) then
          lay = layer(node)
          ECsat = 1.492*cml(node)*theta(node)/thetsl(lay)
          if (ECsat .lt. ECmax) then
            alpsol = 1.0d0
          else
            alpsol = (100.0d0 - (ECsat - ECmax)*ECSLOP) / 100.
            alpsol = max(0.0d0,alpsol)
          endif
        endif

        qrot(node) = qrot(node)*alpwat*alpsol
        QROSUM = qrot(node) + QROSUM
200   continue

500   return
      end

      Subroutine SoilEvap(NumNod,NodeEvapFront,dt,dz,hEvapFront,
     &                    REvaDeep,theta,qRot,
     &                    LAYER,FLGENU,COFGEN,HTABLE,
     &                    THETLO,ALFAMG,THETAR,THETAS,
     &                    Evp,qthetaEF)
******************************************************************
      Implicit None
      include 'param.fi'
*     Input                                                      *
      Integer NumNod,NodeEvapFront,LAYER(macp),THETLO(maho)
      Real*8  dt,REvaDeep,qRot(macp),DZ(macp)
      Real*8  hEvapFront,theta(macp),COFGEN(8,maho)
      Real*8  HTABLE(maho,99),THETAR(macp),THETAS(macp)
      Real*8  ALFAMG(macp)
      Logical FLGENU(maho)
*     Output                                                     *
      Real*8  Evp(macp)
*     Functions                                                  *
      Real*8  theNode
*     Local                                                      *
      Integer Node
      Real*8  qthetaEF,RestREvaDeep    
      Real*8  thetaEF,LimitR8
******************************************************************
      SAVE
c
c     dummy routine
******************************************************************

      Return
      End
C ----------------------------------------------------------------------
      SUBROUTINE SOILTMP (NUMNOD,THETA,THETM1,THETAS,TAV,SWSHF,
     & TEMP,DZ,DISNOD,DT,TAMPLI,TMEAN,DDAMP,TIMREF,DAYNR,Z,
     & FQUARTZ,FCLAY,FORG)
C ----------------------------------------------------------------------
C     Date               : 15/10//97
C     Purpose            : calc. soil temperature profile          
C     Subroutines called : -        
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE 
      INCLUDE 'PARAM.FI'

      INTEGER ErrorCode,I,NODE,NUMNOD,DAYNR,SWSHF

      REAL*8 DISNOD(MACP+1),DT,DZ(MACP),TAV,TAMPLI,TMEAN,DDAMP,TIMREF
      REAL*8 Z(MACP),FCLAY(MACP),FQUARTZ(MACP),FORG(MACP)

      REAL*8 THETA(MACP),THETM1(MACP),HEACAP(MACP),THETAS(MACP)
      REAL*8 HEACND(MACP),HEACON(MACP),TEMP(MACP),TMPOLD(MACP)
      REAL*8 QHBOT,THEAVE(MACP)
      REAL*8 thoma(MACP),thomb(MACP),thomc(MACP),thomf(MACP)

C ----------------------------------------------------------------------

C --- save old temperature profile
      do 10 node = 1,numnod
        TMPOLD(node) = temp(node)
10    continue

      if (SWSHF .eq. 2) then

C --- compute heat conductivity and capacity ---------------------------

        do 110 node = 1,numnod
          THEAVE(node) = 0.5 * (theta(node) + thetm1(node))
  110   continue

C --- calculate nodal heat capacity and thermal conductivity 
        CALL DEVRIES(NUMNOD,THEAVE,THETAS,HEACAP,HEACND,FQUARTZ,
     &        FCLAY,FORG)
        HEACON(1) = HEACND(1)
        do 130 node = 2,numnod
          HEACON(node) = 0.5 * (HEACND(node) + HEACND(node-1))
  130   continue

C --- Calculate new temperature profile --------------------------------

C ---   no heat flow through bottom of profile assumed
        qhbot = 0.0

C ---   calculation of coefficients for node = 1
        I = 1
C ---   temperature fixed at soil surface
        THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
        THOMC(I) = - DT * HEACON(I+1) / (DZ(I) * DISNOD(I+1))
        THOMB(I) = HEACAP(I) - THOMA(I) - THOMC(I)
        THOMF(I) = HEACAP(I) * TMPOLD(I) - THOMA(I) * TAV

C ---   calculation of coefficients for 2 < node < numnod
	  DO 500 I = 2,NUMNOD-1 
          THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
          THOMC(I) = - DT * HEACON(I+1) / (DZ(I) * DISNOD(I+1))
          THOMB(I) = HEACAP(I) - THOMA(I) - THOMC(I)
          THOMF(I) = HEACAP(I) * TMPOLD(I)
500     CONTINUE

C ---   calculation of coefficients for node = numnod
        I = NUMNOD
        THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
        THOMB(I) = HEACAP(I) - THOMA(I)
        THOMF(I) = HEACAP(I) * TMPOLD(I) - (QHBOT * DT)/DZ(I)

C ---   solve for vector temp a tridiagonal linear set
        Call TriDag (Numnod, thoma, thomb, thomc, thomf, Temp,ErrorCode)
        If (ErrorCode.GT.0) Stop 'Error SoilTmp/Tridag'
      else

c ---   analytical solution temperature profile ---
        do 540 i = 1,numnod
          temp(i) = tmean + tampli*(sin(0.0172*(daynr-timref+91.0d0) +
     &              z(i)/ddamp)) / exp(-z(i)/ddamp)
 540    continue

      endif
      
      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE TOTASS (IDAY,DAYL,AMAX,EFF,LAI,KDIF,AVRAD,
     $SINLD,COSLD,DTGA)
C ----------------------------------------------------------------------
C --- Author: Daniel van Kraalingen, 1986
C --- Calculates daily total gross assimilation (DTGA) by performing
C --- a Gaussian integration over time. At three different times of 
C --- the day, irradiance is computed and used to calculate the instan- 
C --- taneous canopy assimilation, whereafter integration takes place.
C --- more information on this routine is given by Spitters et al./1988
C --- Subroutines and functions called: ASSIM, RADIAT
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,IDAY

      REAL*8  LAI,KDIF
      REAL*8  AMAX,AVRAD,COSLD,DAYL,DTGA,EFF,FGROS,GAUSR,HOUR,PARDIF
      REAL*8  PARDIR,SINB,SINLD

      DATA    GAUSR /0.3872983d0/
C ----------------------------------------------------------------------
C --- three point gaussian integration over day
      DTGA  = 0.
      IF (AMAX.LT.1.0d-10) GOTO 20
      DO 10 I=1,3
        HOUR = 12.0d0+DAYL*0.5*(0.5d0+(I-2)*GAUSR)
C --- at a specified hour, diffuse and direct irradiance is computed
        CALL RADIAT (IDAY,HOUR,DAYL,SINLD,COSLD,AVRAD,SINB,PARDIR,
     $              PARDIF)
C --- irradiance and crop properties determine assimilation
        CALL ASSIM (AMAX,EFF,LAI,KDIF,SINB,PARDIR,PARDIF,FGROS)
        IF(I.EQ.2) FGROS=FGROS*1.6
        DTGA = DTGA+FGROS
10    CONTINUE
      DTGA  =DTGA*DAYL/3.6

20    RETURN
      END


C ----------------------------------------------------------------------
      SUBROUTINE WATCON (VOLM1,VOLACT,NUMNOD,THETA,DZ,FMOBIL,LAYER,
     & THETIM,SWCRACK,CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX,NIRD,ACRACK,
     & RAPDRA,DT)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calc. water storage in the soil profile        
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C     Differences SWAP/SWAPS: 
C     - SWAPS: extra parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,LAYER(MACP),SWCRACK
      REAL*8  DZ(MACP),FMOBIL(MACP),CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX
      REAL*8  ACRACK,RAPDRA,DT,NIRD
      REAL*8  THETA(MACP),VOLM1,VOLACT,THETIM(MAHO)

C --- local
      INTEGER i,lay

      save
C ----------------------------------------------------------------------

c --- change of water storage in cracks
      if (SWCRACK.eq.1) then
        ckwm1  = crackw
        crackw = crackw + wtoplat + 
     &           ((arflux+nird)*acrack - qcrack - rapdra) * dt
        crackw = max (0.0d0, crackw)
      endif

C --- update soil profile water storage
      volm1  = volact
      volact = 0.0
      do 10 i  = 1,numnod
        lay    = layer(i)
        volact = volact+(theta(i)+(1.0d0-fmobil(i))*thetim(lay))*dz(i)
 10   continue

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE ZEROCUMU (CQROT,CQDRA,CQBOT,CPTRA,CPEVA,CEVAP,
     & CRUNO,CGRAI,CNRAI,CGIRD,CNIRD,CQDRAR,CQCRACK,
     & CQDRAIN,CQDRD,CSWSUPP,CSWOUT,SQBOT,SQDRA,SQSUR,SQRAP,
     & DECTOT,ROTTOT,SQPREC,SQIRRIG,CSURF)
C ----------------------------------------------------------------------
C     Date               : 9/6/99                                      
C     Purpose            : set cumulative totals to 0                
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      REAL*8  CQROT,CQDRA,CQBOT,CPTRA,CPEVA,CEVAP,CRUNO
      REAL*8  CGRAI,CNRAI,CGIRD,CNIRD
      REAL*8  CQDRAIN(5),CQDRD,cswsupp,cswout,CQDRAR,CQCRACK,CSURF
      REAL*8  SQBOT,SQDRA,SQSUR,SQRAP,DECTOT,ROTTOT,SQPREC,SQIRRIG

C --- local
      INTEGER i
C ----------------------------------------------------------------------
C --- setting cumulative fluxes to zero
      cqrot   = 0.0d0
      cqdra   = 0.0d0
      cqbot   = 0.0d0
      cptra   = 0.0d0
      cpeva   = 0.0d0
      cevap   = 0.0d0
      cruno   = 0.0d0
      CGRAI   = 0.0d0
      CNRAI   = 0.0d0
      CGIRD   = 0.0d0
      CNIRD   = 0.0d0
      cqdrar  = 0.0d0
      cqcrack = 0.0d0

      DO 15 i = 1,5
        cqdrain(i) = 0.0d0
   15 continue

C --- solute fluxes
      SQPREC  = 0.0d0
      SQIRRIG = 0.0d0
      SQBOT  = 0.0d0
      SQDRA  = 0.0d0
      SQSUR  = 0.0d0
      SQRAP  = 0.0d0
      DECTOT = 0.0d0
      ROTTOT = 0.0d0
      CSURF = 0.0d0

C --- surface water system
      cqdrd    = 0.0d0
      cswsupp  = 0.0d0
      cswout   = 0.0d0

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE ZEROINTR (NUMNOD,INQROT,INQ,INQDRA,IQROT,IQDRA,IINTC,
     & IPTRA,IPEVA,IEVAP,IRUNO,IPREC,IQBOT,OUTPER,IQDRAR,IQCRACK,IGRAI,
     & IGIRD,INRAI,INIRD)
C ----------------------------------------------------------------------
C     Date               : 6/6/01                                       
C     Purpose            : set intermediate totals to 0                
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'
 
C --- global
      INTEGER NUMNOD,OUTPER

      REAL*8  INQROT(MACP),INQ(MACP+1),INQDRA(5,MACP),IINTC,IPTRA,IPEVA
      REAL*8  IEVAP,IGRAI,IGIRD,INRAI,INIRD
      REAL*8  IRUNO,IPREC,IQROT,IQDRA,IQBOT,iqdrar,iqcrack
C ----------------------------------------------------------------------
C --- local
      INTEGER node,level
C ----------------------------------------------------------------------

C --- setting intermediate fluxes to zero
      DO 10 NODE = 1,NUMNOD
        INQROT(NODE) = 0.0d0
        INQ(NODE)    = 0.0d0
        DO 20 LEVEL = 1,5
          INQDRA(LEVEL,NODE) = 0.0d0
20      CONTINUE
10    CONTINUE
      INQ(NUMNOD+1) = 0.0d0
      IQROT   = 0.0d0
      IQDRA   = 0.0d0
      IINTC   = 0.0d0
      IPTRA   = 0.0d0
      IPEVA   = 0.0d0
      IEVAP   = 0.0d0
      IRUNO   = 0.0d0
      IPREC   = 0.0d0
      IGRAI   = 0.0d0
      IGIRD   = 0.0d0
      INRAI   = 0.0d0
      INIRD   = 0.0d0
      IQBOT   = 0.0d0
      OUTPER  = 0
      iqdrar  = 0.0d0
      iqcrack = 0.0d0

      RETURN
      END


